/*
Tiedosto: Attribute.java
Luokat:   Attribute

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.attribute;
import converge.attribute.MessageAttributeInterface;
import converge.attribute.ContextModelAttributeInterface;

/**
Attribute on abstrakti yliluokka muille Attribute-sarjan luokille. Yksi Attribute-sarjan aliluokan
ilmentym pit sislln yhden viestin attribuutin (lhettj, otsikko, lhetysaika, trkeys, viestityyppi).
Erityyppisi attribuutteja varten luodaan omia aliluokkia Attributesta, esim. merkkijonoja varten on
luokka AttributeString.
*/
public abstract class Attribute
implements converge.attribute.MessageAttributeInterface,
           converge.attribute.ContextModelAttributeInterface {

    protected Object value;
    protected String name;
    protected String expressionType = "=";

    /**
    * Vertaa olion arvoa annettuun objektiin.
    *
    * @param expressionType vertailutyyppi
    * @param value vertailtava olio
    * @return vertailun tulos
    */
    public abstract boolean compare(String expressionType, Object value);

    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public abstract boolean compare(converge.service.Message msg);

    /**
    * Palauttaa attribuutin tietosislln.
    *
    * @return attribuutin tieto Object-oliona
    */
    public Object getValue() {
        return value;
    }

    /**
    * Asettaa attribuutin tietosislln.
    *
    * @param value attribuutin tieto Object-oliona
    */
    public void setValue(Object value) {
        this.value=value;
    }

    /**
    * Asettaa attribuutin vertailuoperaattorin.
    *
    * @param expressionType vertailuoperaattori, esim. = tai >
    */
    public void setExpressionType(String expressionType) {
        this.expressionType=expressionType;
    }
    
    /**
    * Palauttaa attribuutin vertailuoperaattorin.
    *
    * @return vertailuoperaattori, esim. = tai >
    */
    public String getExpressionType() {
        return this.expressionType;
    }

    /**
    * Palauttaa attribuutin nimen, esimerkiksi Subject
    *
    * @return attribuutin nimi
    */
    public String getName() {
        return this.name;
    }

    /**
    * Asettaa attribuutin nimen, esimerkiksi Subject
    *
    * @param name attribuutin nimi
    */
    public void setName(String name) {
        this.name=name;
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public abstract String[] supportedAttributes();

    /**
    * Muodostaa Attribute*-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public abstract void createXML(org.w3c.dom.Node parent);

    /**
    * Muodostaa annetusta XML-elementist Attribute*-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public abstract boolean loadXML(org.w3c.dom.Node myContent);

    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public abstract String getDescription();

}
