/*
Tiedosto: AttributePriority.java
Luokat:   AttributePriority

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti
  18.12.2002  Toiminnallisuutta
*/

package converge.attribute.attributeImplementations;

import converge.kernel.Log;
import converge.service.Message;
import converge.attribute.Attribute;
import org.w3c.dom.*;

/**
* Attribute-luokan aliluokka, joka osaa ksitell viestin mukana tullutta
* Priority-arvoa (1,2,3,4,5)
*/
public class AttributePriority extends Attribute {

    private Message myMsg;
    private Integer myValue = new Integer(3);
    private static String[] supported = { "X-Priority" };
    private static String description = "Viestin Priority-kentn ksittely (kiireellisyys)";

    /**
    * Vertaa olion arvoa annetulla tavalla annettuun objektiin.
    *
    * @param expType vertailutyyppi
    * @param compareValue vertailtava olio
    * @return vertailun tulos
    */
    public boolean compare(String expType, Object compareValue) {
        parseMyValue();
        if(compareValue == null || myValue == null)
            return false;
        Integer givenValue = new Integer(3);
        try {
            compareValue = ((String)compareValue).replaceAll("\\D","");
            if(compareValue.equals(""))
                compareValue="3";
            givenValue = new Integer((String)compareValue);
            if(givenValue.intValue()<1)
                givenValue = new Integer("1");
            else if(givenValue.intValue()>5)
                givenValue = new Integer("5");

        } catch(Exception ex) {
            Log.stack(ex,"AttributePriority");
            return false;
        }

        try {
            if (expType.equals("="))
                return this.myValue.compareTo(givenValue) == 0;
            if (expType.equals("!="))
                return this.myValue.compareTo(givenValue) != 0;
            if (expType.equals("<"))
                return this.myValue.compareTo(givenValue) < 0;
            if (expType.equals(">"))
                return this.myValue.compareTo(givenValue) > 0;
        } catch(Exception e) {
            return false;
        }
        return false;
    }

    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public boolean compare(converge.service.Message msg) {
        parseMyValue();
        if(myValue == null)
            return false;
        if(this.name == null)
            this.name=supported[0];
        try {
            Attribute a = (Attribute)msg.getMessageAttribute(this.name);
            if(a == null)
                return false;
            return a.compare(this.expressionType,myValue.toString());
        } catch(Exception e) {
            return false;
        }
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public String[] supportedAttributes() {
        return supported;
    }

    /**
    * Muodostaa AttributePriority-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        if(parent==null)
            return;
        parseMyValue();

        Document doc;
        if(parent.getOwnerDocument() == null)
            doc = (Document)parent;
        else
            doc = parent.getOwnerDocument();

        Element apri = doc.createElement("AttributePriority");
        parent.appendChild(apri);

        Element name = doc.createElement("name");
        name.appendChild(doc.createTextNode(this.name));
        apri.appendChild(name);

        Element ex = doc.createElement("expression");
        ex.appendChild(doc.createTextNode(this.expressionType));
        apri.appendChild(ex);

        Element value = doc.createElement("value");
        value.appendChild(doc.createTextNode(this.value.toString()));
        apri.appendChild(value);
    }

    /**
    * Muodostaa annetusta XML-elementist AttributePriority-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        if ( myContent == null || !myContent.getNodeName().equals("AttributePriority") )
            return false;
        try {
            Node curr = myContent.getFirstChild();
            while(curr != null) {
                if(curr.getNodeName().equals("name")) {
                    this.name = curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("value")) {
                    this.value=curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("expression")) {
                    this.expressionType=curr.getFirstChild().getNodeValue();
                }

                curr=curr.getNextSibling();
            }
        } catch(Exception ex) {
            Log.output("AttributePriority").println("XML lataus ei onnistunut");
            Log.output("AttributePriority").println(ex.getMessage());
            return false;
        }
        return this.name != null && this.value != null;
    }

    /**
    * Tarkistaa olion sisltmn tietoarvon ja muuttaa sen normaalimuotoon
    */
    private void parseMyValue(){
        if(this.value == null) {
            this.value="3"; // oletusarvo, "normaali"
            myValue=new Integer("3");
        }
        try{
            myValue = new Integer(((String)this.value).replaceAll("\\D",""));
            if(myValue.intValue()<1)
                myValue = new Integer("1");
            else if(myValue.intValue()>5)
                myValue = new Integer("5");
        } catch(Exception ex) {
            Log.stack(ex,"AttributePriority");
        }
    }

    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public String getDescription() {
        return description;
    }

    /**
    * Asettaa attribuutin sisltmn viestiolion viitteen
    *
    * @param msg Jrjestelmn sisinen viesti
    */
    public void setMessage( Message msg) {
        this.myMsg = msg;
    }

    /* testimetodi */
    public static void main(String[] args) {
        AttributePriority a = new AttributePriority();
        a.setValue("3"); //normal
        System.out.println(" 3 = 3 => " + a.compare("=","3"));
        System.out.println(" 3 = 2 => " + a.compare("=","2"));
        System.out.println(" 3 = xxx => " + a.compare("=","xxx"));
        System.out.println(" 3 > 4 => " + a.compare(">","4"));
        System.out.println(" 3 < 4 => " + a.compare("<","4"));
        a.setValue("xxx"); //virhe, vastaa kolmosta
        System.out.println(" xxx = 3 => " + a.compare("=","3"));
        System.out.println(" xxx > 3 => " + a.compare(">","3"));
        System.out.println(" xxx > 4 => " + a.compare(">","4"));
        System.out.println(" xxx < 4 => " + a.compare("<","4"));
        System.out.println(" xxx = 4 => " + a.compare("=","4"));

        a.setValue("1"); //trkein
        System.out.println(" 1 = 1 => " + a.compare("=","1"));
        System.out.println(" 1 > 1 => " + a.compare(">","1"));
        System.out.println(" 1 > 4 => " + a.compare(">","4"));
        System.out.println(" 1 < 4 => " + a.compare("<","4"));
        System.out.println(" 1 = 4 => " + a.compare("=","4"));
        System.out.println(" 1 < 0 => " + a.compare("<","0"));
        System.out.println(" 1 > 6 => " + a.compare(">","6"));
        System.out.println(" 1 = 0 => " + a.compare("=","0"));

        /* tulostaa:
        3 = 3 => true       ok
        3 = 2 => false      ok
        3 = xxx => true     xxx laiton, muutetaan 3:ksi
        3 > 4 => false      ok
        3 < 4 => true       ok
        xxx = 3 => true     xxx laiton, muutetaan 3:ksi
        xxx > 3 => false    -||-
        xxx > 4 => false    -||-
        xxx < 4 => true     -||-
        xxx = 4 => false    -||-
        1 = 1 => true       ok
        1 > 1 => false      ok
        1 > 4 => false      ok
        1 < 4 => true       ok
        1 = 4 => false      ok
        1 < 0 => false      0 laiton, muutetaan 1:ksi
        1 > 6 => false      6 laiton, muutetaan 5:ksi
        1 = 0 => true       0 laiton, muutetaan 1:ksi
        */
    }

}
