/*
Tiedosto: AttributeString.java
Luokat:   AttributeString

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti
  18.12.2002  Virhetilanteiden ksittely

*/

package converge.attribute.attributeImplementations;

import converge.kernel.Log;
import converge.service.Message;
import converge.attribute.Attribute;
import org.w3c.dom.*;
import converge.kernel.Log;

/**
* Attribute-luokan aliluokka, joka osaa ksitell merkkijonoja.
*/
public class AttributeString extends Attribute{

    private Message myMsg;
    private String myValue = "";
    private String description = "Tekstikentn arvon vertailuun tarkoitettu";
    private String[] supported = { "Subject", "Message-ID", "From", "MESSAGE_BODY", "To", "CC" };

    /**
    * Vertaa olion arvoa annettuun objektiin.
    *
    * @param expType vertailutyyppi
    * @param compareValue vertailtava olio
    * @return vertailun tulos
    */
    public boolean compare(String expType, Object compareValue) {
        parseMyValue();
        if(myValue == null || compareValue == null)
            return false;

        String givenValue = "";
        try {
            givenValue = (String)compareValue;
        } catch(Exception ex){
            Log.stack(ex,"AttributeString");
            return false;
        }

        try {
            if (expType.equals("="))
                return this.myValue.compareTo(givenValue) == 0;
            if (expType.equals("!="))
                return this.myValue.compareTo(givenValue) != 0;
            if (expType.equals("<"))
                return this.myValue.compareTo(givenValue) < 0;
            if (expType.equals(">"))
                return this.myValue.compareTo(givenValue) > 0;

            if (expType.equals("in")) {
                if (this.myValue.indexOf(givenValue) >= 0) {
                    return true;
                }
              return false;
            }
            if (expType.equals("!in")) {
                if (this.myValue.indexOf(givenValue) >= 0)
                    return false;
                return true;
            }
        } catch(Exception e) {
            return false;
        }
        return false;
    }

    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public boolean compare(converge.service.Message msg) {
        if(this.value == null)
            return false;
        parseMyValue();
        if(myValue == null)
            return false;
        if(this.name == null)
            this.name=supported[0];
        try {
            Attribute a = (Attribute)msg.getMessageAttribute(this.name);
            if(a == null)
                return false;
            return a.compare(this.expressionType,(String)myValue);
        } catch(Exception e) {
            return false;
        }
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public String[] supportedAttributes() {
        return supported;
    }

    /**
    * Muodostaa AttributeString-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        if(parent==null)
            return;
        parseMyValue();

        Document doc;
        if(parent.getOwnerDocument() == null)
            doc = (Document)parent;
        else
            doc = parent.getOwnerDocument();

        Element as = doc.createElement("AttributeString");
        parent.appendChild(as);

        Element name = doc.createElement("name");
        name.appendChild(doc.createTextNode(this.name));
        as.appendChild(name);

        Element ex = doc.createElement("expression");
        ex.appendChild(doc.createTextNode(this.expressionType));
        as.appendChild(ex);

        Element value = doc.createElement("value");
        value.appendChild(doc.createTextNode(this.value.toString()));
        as.appendChild(value);
    }

    /**
    * Muodostaa annetusta XML-elementist AttributeString-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        if ( myContent == null || !myContent.getNodeName().equals("AttributeString") )
            return false;
        try {
            Node curr = myContent.getFirstChild();
            while(curr != null) {
                if(curr.getNodeName().equals("name")) {
                    this.name = curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("value")) {
                    this.value=curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("expression")) {
                    this.expressionType=curr.getFirstChild().getNodeValue();
                }

                curr=curr.getNextSibling();
            }
        } catch(Exception ex) {
            Log.output("AttributeString").println("XML lataus ei onnistunut");
            Log.output("AttributeString").println(ex.getMessage());
            return false;
        }
        return this.name != null && this.value != null;
    }


    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public String getDescription(){
        return description;
    }

    /**
    * Tarkistaa olion sisltmn tietoarvon ja muuttaa sen normaalimuotoon
    */
    private void parseMyValue(){
        try {
            myValue = (String)this.value;
        } catch(Exception ex) {
            Log.stack(ex,"AttributeString");
        }
    }

    /**
    * Asettaa attribuutin sisltmn viestiolion viitteen
    *
    * @param msg Jrjestelmn sisinen viesti
    */
    public void setMessage(Message msg) {
        this.myMsg = msg;
    }
    
    /* testimetodi */
    public static void main(String[] args) {
        AttributeString a = new AttributeString();
        a.setValue("foo");
        System.out.println("foo compareto bar => " + "foo".compareTo("bar"));
        System.out.println("bar compareto foo => " + "bar".compareTo("foo"));
        System.out.println(" foo = foo => " + a.compare("=","foo"));
        System.out.println(" foo < foo => " + a.compare("<","foo"));
        System.out.println(" foo > foo => " + a.compare(">","foo"));
        System.out.println(" foo != foo => " + a.compare("!=","foo"));
        System.out.println(" foo < bar => " + a.compare("<","bar"));
        System.out.println(" foo > bar => " + a.compare(">","bar"));
        System.out.println(" foo in foo => " + a.compare("in","foo"));
        System.out.println(" foo !in foo => " + a.compare("!in","foo"));
        System.out.println(" foo in bazfoobar => " + a.compare("in","bazfoobar"));
        System.out.println(" foo !in bazfoobar => " + a.compare("!in","bazfoobar"));
        a.setValue("bazfoobar");
        System.out.println(" bazfoobar in foo => " + a.compare("in","foo"));
        System.out.println(" bazfoobar !in foo => " + a.compare("!in","foo"));
        /*

        foo = foo => true
        foo < foo => false
        foo > foo => false
        foo != foo => false
        foo < bar => false
        foo > bar => true
        foo in foo => true
        foo !in foo => false
        foo in bazfoobar => false
        foo !in bazfoobar => true
        bazfoobar in foo => true
        bazfoobar !in foo => false
        */

    }
}
