/*
Tiedosto: ClientManager.java
Luokat:   ClientManager

Tekij:   Joni Karppinen
Luonti:   25.11.2002

Muutokset:  

3.12.   Joni    
        Muutettu vastaamaan suunnitteludokumenttia ja viimeisint ksityst 
        asiakasmoduulin rakenteesta.
4.12.   Joni
        Koodattu useimpien metodien sislt        
17.12.  Joni
        Listty kommentteja, Javadocceja ym.        
*/

package converge.clientmanager;
import converge.kernel.*;
import converge.service.*;
import java.util.*;
import java.io.*;

/**

ClientManager-luokasta on olemassa yksi ilmentym, joka toimii vlittjn
ytimen ja  erilaisten asiakasohjelmien/ptelaitteiden vlill. 
<br><br>
Tss toteutuksessa ClientManager hoitaa sessionhallintaa, sisn- ja uloskirjautumista sek vlitt User-olioita asiakasohjelmaluokille (webcore-luokille). 
<br><br>

*/ 
public class ClientManager{    
    
    /**
    Luokan latauksen yhteydess asetukset luetaan tiedostosta thn muuttujaan
    */
    private static Properties settings = new Properties();
    
    /**
    Session vanhentumisaika (millisekunteina) 
    */
    private static long expirationTime;
    
    /** Viitett ainoaan ClientManager-olioon silytetn tss 
    luokkamuuttujassa.
    */
    private static ClientManager cm;

    /**
    Sislt tiedon siit onko ClientManagerin shutdown-metodia kutsuttu
    */
    private boolean stop = false;
    
    /**    
    Tietorakenne, jossa (aktiiviset) session-id:t kuvataan vastaamaan
    User-oliota ja aikaleimaa.
    <br><br>
    long session-id  ->  SessionEntry (User, timestamp)
    */    
    private HashMap activeSessions;    

    /**
    Tietorakenne, jossa (aktiivisia) user-olita vastaa ClientHandler -olio.
    Eli kuvaus:  User ->  ClientHandler   s.e. ClientHandler \in clientHandlers    
    <br><br>
    [Ainakin viestien (push-tyyppist) lhettmist varten tarvittaisiin tllainen
    rakenne. Prototyypiss on kuitenkin vain www-asiakas, jolle ei voi aktiivisesti
    lhett mitn, joten tt tietorakennetta ei kytet. Tydellisemmss toteutuksessa 
    tietorakennetta pitisi pivitt ainakin login ja logout -metodeissa.]    
    */    
    private HashMap activeUsers;
        
    
    /**
    Sisluokka jota kytetn tietorakenteessa, jossa session-id:t  
    vastaa User-olio ja aikaleima.
    */        
    private class SessionEntry{
        private User user;
        private long timeStamp;  

        private SessionEntry(User u) {
              user = u;
              timeStamp = System.currentTimeMillis();
        }
    }
        
    /**
    Aktiiviset/olemassaolevat/tuetut ptelaitehandlerit
    <br><br>
    (Tarvitaan lista olemassaolevista erilaisista client-rajapinnoista    
    Kutakin tyyppi (esim. www-clienthandler) on vain yksi olio joka hoitaa kaikkien ko. ptelaitetta kyttvien kyttjien palvelemisen.)    
    */    
    //ClientHandler[] clientHandlers;
    private ArrayList clientHandlers;
    
    /**
    Viite jrjestelmn ainoaan UserManager-olioon
    */
    private UserManager userManager;
    
        
    /**
    Lohko jossa suoritetaan tiettyj alustustoimintoja kun luokka ladataan muistiin.    
    
    Ladataan asetuksia tiedostosta config/clientmanager.cfg.
    (Tss tapauksessa vain yksi asetus.)    
    */    
    static{        
        try{            
            settings.load(new FileInputStream(new File("config/clientmanager.cfg")));
            expirationTime = Long.parseLong(         
                             settings.getProperty("session-expiration-time", "3600") 
                                             );
            expirationTime *= 1000;        // sekunneista millisekunneiksi
                            
        }
        catch(Exception e){
            System.out.println("CLIENTMANAGER-alustuslohko: " +e);
            // ajaksi tunti jos asetustiedoston luku ei onnistunut:
            expirationTime = 3600000;    
        }        

    }
    
        
    /**
    Konstruktori, tt kutsutaan vain kerran jrjestelmn kynnissoloaikana.
    
    Sislt joitakin alustustoimenpiteit    
    */
    private ClientManager(){    
        activeSessions = new HashMap();
        activeUsers = new HashMap();        
        clientHandlers = new ArrayList();        
         
        clientHandlers.add(WebClientHandler.getInstance());
        // .
        // . 
        
        // laajemmassa toteutuksessa nm voisi lukea jostain asetustiedostosta tms.
            
        // haetaan viitteet tunnettuihin ytimen osiin
        userManager = UserManager.getInstance();             
    }
    

//metodit, jotka tarjotaan ClientHandlereille

    /**
    Palauttaa annettua sessionId:t vastaavan User-olion. Jos sessionId ei ole
    validi (esimerkiksi istunto on vanhentunut) tai yllpitj on poistanut
    kyttjn jrjestelmst, metodi palauttaa null-arvon.
<br><br>
    Istunnon vanhentumisaika voidaan asettaa asetustiedostossa config/clientmanager.cfg. Oletusarvona on yksi tunti. Aina kun tt metodia onnistuneesti kutsutaan, uusitaan istunnon aikaleima. 
    
    @param sessionId yksilllinen sessiotunnus
    @return tunnusta vastaava kyttj tai null  
    */
    public synchronized User getUserBySessionId(long sessionId){
        // todo (paremmassa toteutuksessa):   
        // optimointia? onko esim. System.currentTimeMillis liian raskas/hidas operaatio?
        
        if (this.stop)
            return null;
            
        SessionEntry entry = (SessionEntry) activeSessions.get(new Long(sessionId));
        
        if (entry == null)    // (jostain syyst) session-id:t ei ollut tietorakenteessa
            return null;

        // tarkistetaan onko sessio vanhentunut        
        // (vanhentumisaika on luettu asetustiedostosta expirationTime -muuttujaan)         
            
        if ((entry.timeStamp + expirationTime) < System.currentTimeMillis()){            
            // poistetaan session-id tietorakenteesta
            activeSessions.remove(new Long(sessionId));
            return null;
        }
        
        // tarkistetaan onko esim. yllpitj poistanut kyttjn jrjestelmst        
        if (entry.user.isDeleted())
            return null;        
        
        //Log.output("ClientManager").println("user-object succesfully "
        //    +"fetched: "+entry.user.getName()");        
        
        entry.timeStamp = System.currentTimeMillis(); // uusitaan aikaleima        
        return entry.user;        
    }


    /** 
    Tt metodia kutsutaan sisnkirjautumisen yhteydess. Jos annetut username
    ja password ovat validit, palautetaan long-tyyppi oleva session-id.
    
    <br><br>Jos tunnus ja salasana eivt vastaa toisiaan, tai tunnusta ei ole olemassa,
    tm metodi palauttaa virhekoodina negatiivisen luvun (-1).

    <br><br>
    Jos parametri createNew on true, yritetn luoda uusi kyttjtunnus. Jos taas
    parametri group on true, yritetn kirjautua sisn "ryhmn" (eli kirjautua
    muuttamaan ryhmn asetuksia).    
    <br><br>
    Tm metodi on toteutettu niin, ett se kutsuu ytimen UserManagerin palveluita ja
    (jos tunnus ja salasana ovat oikein) saa sielt User-olion, jota vastaamaan se
    luo uuden session-id:n.

    @param username kyttjtunnnus
    @param password kyttjtunnusta vastaava salasana
    @param createNew luodaanko uusi kyttjtunnus
    @param group kirjaudutaanko sisn muuttamaan ryhmn asetuksia    
    @return uusi, yksiksitteinen session-id
    */    
    public synchronized long login(String username, String password, 
                                   boolean createNew, boolean group){
                                       
        // tarkistetaan virhetilanteen varalta
        if (this.stop || username == null || password == null)
            return -1;

        User newUser = null;;
        long newSessionId;                
    
        // 1) User-olion haku / luonti 
        
        if (!createNew && !group)        
            newUser = userManager.getUser(username, password);
        else if (group)
          newUser = userManager.getGroup(username, password);
            else
                newUser = userManager.newUser(username, password);            
            
        // jos tunnus/salasana vrin (tai luonti eponnistui), 
        // saadaan null UserManagerilta       
        if (newUser == null){            
            Log.output("ClientManager").println(
                "Unsuccessful login attempt with user name \"" + username +"\"");
            return -1;
        }
        
        // 2) session-id:n luonti
        // tss vaiheessa riittnee ihan vaan System.currentTimeMillis()
        // hienostuneempia menetelmi voi kehitt myhemmin

        newSessionId = System.currentTimeMillis();
        
        //  esimerkki paremmasta tavasta:        
        //          Integer.toHexString(("koe" + new Date()).hashCode()) + "-"
        //          + Integer.toHexString((int)(Math.random()*100000000));


        // 3) session-id:n ja User-olion lisminen tietorakenteeseen
            
        activeSessions.put(new Long(newSessionId), new SessionEntry(newUser));
        
        Log.output("ClientManager").println("User \"" + newUser.getName() 
                         + "\" successfully logged in");
        
        return newSessionId;    
    }

    /**     
    Asiakasohjelmaluokat kutsuvat tt metodia, kun kyttj haluaa
    eksplisiittisesti kirjautua ulos palvelusta. Parametrina annettu sessionId ja
    sit vastaava User-olio poistetaan ClientManagerin sisisest
    tietorakeenteesta.
    <br><br>
    (Eli jos samalla sessionId:ll kutsuttaisiin tmn jlkeen
    getUserBySessionId-metodia, palauttaisi se arvon null.) 

    @param sessionId session-id, joka invalidoidaan
    */    
    public synchronized void logout(long sessionId){
        if (this.stop)
            return;
        
        activeSessions.remove(new Long(sessionId));                    
    }
    
    
    
// metodit, jotka tarjotaan ytimelle

    /**
    Viestin lhetys kyttjlle. Tmn kutsuminen ei takaa viestin perillemenoa; se
    riippuu mm. kytetyst ptelaitteesta.                  
<br><br>
    (Vastaanottaja saadaan Message-olion user-kentst.)
<br><br>
    [Tt metodia ei kytet tss prototyypiss]
    
    @param msg Lhetettv viestiolio
    */ 
    public void sendMessage(Message msg){    
        
        // kutsutaan siis jonkin ClientHandlerin sendMessagea
        // oikea ClientHandler saadaan activeUsers -HashMapista
        ClientHandler clientHandler = (ClientHandler) activeUsers.get(msg.getUser());
        
        if (clientHandler != null)
            clientHandler.sendMessage(msg);
            
        }
    

    /** 
    ClientManagerin kynnistys. Kutsuu jokaisen ClientHandlerin start-metodia.
    <br><br>
    Prototyypiss aiheuttaa webcore-HTTP-palvelimen kynnistymisen.    
    */
    public void start(){        
        // kydn lpi clientHandlers-lista ja kutsutaan jokaisen kohdalla start()         
        Iterator i = clientHandlers.iterator();
        while (i.hasNext()){
            ClientHandler c = (ClientHandler) i.next();
            c.start();
        }                           
    }

    /**
     ClientManagerin alasajo. Tm metodi kutsuu kaikkien olemassaolevien
    ClientHandlereiden shutdown-metodeja.     
    */    
    public void shutdown() {
        this.stop = true;
        Iterator i = clientHandlers.iterator();
        while (i.hasNext()){
            ClientHandler c = (ClientHandler) i.next();
            c.shutdown();
        }         
    }    

    /**
    Palauttaa viitteen jrjestelmn ainoaan ClientManager -olioon.
    
    @return jrjestelmn ainoa ClientManager-ilmentym
    */
    public static synchronized ClientManager getInstance(){
        if (ClientManager.cm == null){            
            ClientManager.cm = new ClientManager();             
            cm.start();        // kynnistetn ClientManager
        } 
        if (cm.stop)
            return null;
        return (ClientManager.cm);
    }

}


