/*

Tiedosto   : ActionLoader.java
Luokat     : ActionLoader

         
Tekij     : Olli Pettay
Luonti pvm : 25.11.2002


Muutokset :

25.11.2002 Olli Pettay
           Luotu
07.12.2002 Mikko Hiipakka
           Toiminta rakennettu

*/


package converge.kernel;

import converge.kernel.Log;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;


/**
*ActionLoader lataa jrjestelmn kaikki tunnetut ActionInterfacen toteuttavat luokat.
*/
public class ActionLoader{
  
  private static String packageName      = "converge.kernel.actionImplementations.";
  private static final String actionDir  = "converge.action.dir";
  private static String defaultActionDir = "converge"+ File.separator 
                                                     + "kernel" 
                                                     + File.separator
                                                     + "actionImplementations"
                                                     + File.separator;

  // Toteutusluokkat tarjoamiensa toimintojen loogisten nimien mukaan mpttyn
  private static HashMap actions  = new HashMap();
  

  /**
   * Palveluna luodaan ActionInterface ilmentym, joka
   * tarjoaa toteutuksen "name"-nimiselle toiminnolle
   * @param name Pyydetyn toiminnon nimi
   * @return Palauttaa uuden ActionInterface ilmentymn, ja
   * jos luonti eponnistuu palauttaa null
  */   
  public static ActionInterface newActionInstance(String name) {
   //DEBUG******************************************************************
    Log.output("ActionLoader").println("ACTIONLOADER-newActionInstance : Ladataan uusi ilmentym");
    //DEBUG******************************************************************
    Class aClass = (Class)actions.get(name);
    ActionInterface toReturn = null;
    
    if (aClass != null) {
      try{
        toReturn = (ActionInterface)aClass.newInstance();
      }catch( Exception ex ){
        Log.stack( ex );
        //DEBUG******************************************************************
        Log.output("ActionLoader").println("ACTIONLOADER-newActionInstance : Virhe luomisessa " + ex.getMessage() );
        //DEBUG******************************************************************
      }
    }
    return toReturn;
  }
  
  
  /**
   * Ladataan Action-toteutusluokat, ja tarkastetaan niiden tarjoamat 
   * toteutukset.
  */     
  public static synchronized void loadClasses() {
    //DEBUG******************************************************************
    Log.output("ActionLoader").println("ACTIONLOADER-loadClasses : Aloitetaan luokkien lataus");
    //DEBUG******************************************************************
    actions = new HashMap();
    String pathToActions = System.getProperty(actionDir, defaultActionDir);
    File dir = new File(pathToActions);
    if (!dir.isDirectory())
      return; 
      
    File[] classes = dir.listFiles(new FilenameFilter(){
              public boolean accept(File dir, String name) {
                if (name.endsWith(".class")) 
                  return true;
                return false;
              }
            });
            
    for (int i = 0; i < classes.length; ++i) {
      String name = classes[i].getName();
      String classname = name.substring(0,name.indexOf(".class"));
      //DEBUG******************************************************************
      Log.output("ActionLoader").println("ACTIONLOADER-loadClasses : Ladataan Luokka : " + classname);
      //DEBUG******************************************************************
      try{ 
        Class aClass = Class.forName( packageName + classname);
        setActions( aClass );
      }catch( Exception ex ){
        Log.stack( ex );
      }
    }
  }
  
  private static void setActions( Class aClass ) {
    ActionInterface ai = null; 
    try{
      ai = (ActionInterface)aClass.newInstance();
    }catch( Exception ex ){
      Log.stack( ex );
    }
    String[] supported = ai.supportedActions();

    for(int i = 0; i < supported.length; ++i) {
      if (ai instanceof ActionInterface) {
        if ( !actions.containsKey(supported[i])){
            //DEBUG******************************************************************
            Log.output("ActionLoader").println("ACTIONLOADER-setActions : Tuettu toiminto : " + supported[i]);
            //DEBUG******************************************************************
            actions.put(supported[i], aClass);
        }
        else{ 
          Class old = (Class)actions.get( supported[i] );
          Log.output("ActionLoader").println("REDEFINING ActionInterface:"
                                             + supported[i] + ".  " 
                                             + aClass.getName() + " : "
                                             + old.getName() );
                                             
        }
      }
    }
  }
    
  public static String[] getKnownActions() {
    return (String[]) actions.keySet().toArray(new String[0]);
  }
  
//DEBUG************************************************************************
  public static void main(String[] args){
    ActionLoader.loadClasses();
    ActionInterface mai = newActionInstance( "Kokeilu" );
    if ( mai == null ){
      System.out.println("ACTIONLOADER-main : Action \"Kokeilu\", ei ladattu");
      return;
    }
    String[] supported = mai.supportedActions();
    System.out.println("");
    System.out.println("");
    System.out.println("ACTIONLOADER-main : Testi Attribuutti tukee");
    for(int i = 0; i < supported.length; ++i) 
      System.out.println( i + ". tuettu : " + supported[i]);
    
    String[] all = getKnownActions();
    System.out.println("ACTIONLOADER-main : Known actions");
    for (int i = 0; i < all.length; ++i)
      System.out.println(i + ". action : " + all[i]);
  }
//DEBUG************************************************************************
}
