package converge.kernel;

import org.w3c.dom.*;

/**
*Consequence-oliolla kuvataan yht sntjen toiminnoissa mritelty
*tapahtumaa.
*/
public class Consequence implements XMLContentInterface{

  private int type;
  private String name = null;
  private String value = null;
  public Consequence(){}
  public Consequence(int type, String name, String value) {
    this.type = type;
    this.name = name;
    this.value = value;
  
  }
  
/**
*Tapahtuman tyyppi.
*
*@return joko Rule.EVENT_ADD tai Rule.EVENT_DELETE
*/
  public int getType() {
    return this.type;
  }
  
/**
*Tapahtuman nimi.
*
*@return palautetaan nimi
*/
  public String getName() {
    return this.name;
  }

/**
*Tapahtuman arvo
*
*@return palautetaan arvo
*/
  public String getValue() {
    return this.value;
  }
  
/**
*Luodaan Java-koodi
*
*@return palautetaan koodi merkkijonona
*/
  public String makeCode(){
    String code = "\nevent";
    if (type == Rule.EVENT_ADD) {
      code += ".addEvent(\"" + this.name + "\",\"" + this.value + "\");\n";
    }
    else if (type == Rule.EVENT_DELETE) {
      if (this.value == null)
        code += ".deleteEvent(\"" + this.name + "\");\n";
      else {
        code += ".deleteEvent(\"" + this.name + "\",\"" + this.value + "\");\n"; 
      }
    }
    else {
      code = "";
    }
    return code;
  }

  
/**
*Luodaan XML
*/
  public void createXML(Node par){
    Element parent = (Element)par;
    Document dom = parent.getOwnerDocument();
    Element el = dom.createElement("consequense");
    el.setAttribute("name", this.name);
    el.setAttribute("type", new Integer(this.type).toString());
    if (value != null)
      el.setAttribute("value", this.value);
    
    parent.appendChild(el);
  }

/**
*Ladataan XML.
*/
  public boolean loadXML(Node cont){
    Element content = (Element)cont;
    Document dom = cont.getOwnerDocument();
    if (!content.getTagName().equals("consequense"))
      return false;
    this.name = content.getAttribute("name");
    this.type = new Integer(content.getAttribute("type")).intValue();
    if (content.hasAttribute("value"))
      this.value = content.getAttribute("value");
    return true;
  }

}


