/*

Tiedosto   : ContextModel.java
Luokat     : ContextModel

         
Tekij     : Mikko Hiipakka
Luonti pvm : 17.11.2002


Muutokset :

17.11.2002  Mikko Hiipakka
            Luokan rakenne luotu ja pseudokoodilla suunniteltu toimintaa
5.12.2002   Mikko Hiipakka
            Toiminta rakennettu            
9.12.2002   Mikko Hiipakka
            Toimintaa tarkennettu ja testattu ( ei startProcessing )
15.12.2002  Mikko Hiipakka 
            Rakenne tarkistettu
*/

package converge.kernel;

import converge.service.Message;
import converge.kernel.XMLContentInterface;
import converge.attribute.*;
import converge.attribute.attributeImplementations.ContextModelValueAttribute;
import java.util.ArrayList;
import org.w3c.dom.*;

/**
 *Tll luokalla kuvataan kyttjien ja ryhmien mrittelemi kontekstimalleja.
 *
*/
public class ContextModel implements XMLContentInterface{

	
  private ArrayList forcedAttributes;
  private ArrayList attributes;
  private String name;


  /**
   * Alustaa uuden Kontekstimalli ilmentymn valmiiksi, attribuuttien lismiselle
   * @param name Kontekstimallille annettava nimi
  */
  public ContextModel( String name ){
    attributes = new ArrayList();
    forcedAttributes = new ArrayList();
    this.name = name;
  }

  /**
   * Konstruktori, jolla kontekstimalli ilmentym osaa muodostaa tietosisltns annetusta 
   * XML elementist
   * @param contextModelElement org.w3c.dom.Element
  */
  public ContextModel( Node contextModelElement ) throws Exception{
    this( contextModelElement.getNodeName() );
    if (!this.loadXML( contextModelElement )){
      throw new Exception("CONTEXTMODEL : XML alustus eponnistui");
    }
  }


  /**
   * Uuden attribuutin liittminen kontekstimalliin. Vertailu suoritetaan 
   * "Viestiolion.Attribuutin.Arvo" experssionType value esim. vertailu arvo &lt; value
   * @param name Liitettvn attribuutin nimi
   * @param expressionType Laskusnnt "&lt;","&gt;","="
   * @param value Asetettavat raja-arvot
   * @param forced Onko attribuutti pakotettu
   * @param weight painoarvo
  */
  public void addAttribute( String name, String expressionType, Object value, boolean forced , int weight) {
    ContextModelAttributeInterface attr = AttributeLoader.newContextModelAttributeInstance( name );
    if( attr == null ){
      Log.output("ContextModel-Attributes").println("Attribuutin lisys eponnistui - " 
                                      + "AttributeLoader palautti null ( attribute : " + name);
      return;
    }
    attr.setName( name );
    attr.setValue( value );
    attr.setExpressionType( expressionType );
    if( forced )
      forcedAttributes.add( new AttributeWeightPair( attr, weight ) );
    else
      attributes.add( new AttributeWeightPair( attr, weight ) );
  }


  public boolean loadXML( Node receivedContent ){
    Element myContent = null;
    try{
      myContent = (Element)receivedContent;
    }catch( Exception ex ){
      Log.output("ContextModel").println("XML latautaus eponnistui - " 
                      + "Annettun Node muutos Elementiksi ei onnistunut");
      Log.output("ContextModel").println( ex.getMessage() );
      Log.stack( ex );                      
      return false;
    }
    if ( !(myContent.getTagName()).equals("ContextModel") ){
      Log.output("ContextModel").println("XML lataus eponnistui - " 
                  + "Vastaanotettu elementti ei ollut <ContextModel>");
    }
    myContent = (Element)myContent.getFirstChild();
    this.name = myContent.getTagName();
    NodeList attrTypes = myContent.getChildNodes();
    boolean success = true;

    for( int i=0; i < attrTypes.getLength(); i++ ){
      if ( !success )
        break;
      //Kaikki forcedAttributes listalle kuuluvat
      if ( ( ((Element)attrTypes.item(i)).getTagName() ).equals("ForcedAttributes") )
        success = loadAttributes( attrTypes.item(i).getChildNodes(), forcedAttributes );
      //Kaikki attributes listalle kuuluvat  
      if ( ( ((Element)attrTypes.item(i)).getTagName() ).equals("Attributes") )
        success = loadAttributes( attrTypes.item(i).getChildNodes(), attributes );
    }
    return success;
  }


  
  public void createXML( Node parent ){
    Document myDom = null;
    try{
      myDom = (Document)parent;
    }catch( Exception ex ){
      Log.output("ContextModel").println("XML luonti eponnistui - " 
                      + "Annettun Node muutos Documentiksi ei onnistunut");
      Log.output("ContextModel").println( ex.getMessage() );
      Log.stack( ex );                      
      return;
    }

    Element myParent = myDom.createElement("ContextModel");
    myDom.appendChild( myParent );
    Element myXML = myDom.createElement( this.name );
    myParent.appendChild( myXML );
    if ( forcedAttributes.size() > 0 ){  
      Element forcedChildren = myDom.createElement( "ForcedAttributes" );
      myXML.appendChild( forcedChildren );
      this.createXMLHelp( forcedAttributes, forcedChildren );
    }
    if ( attributes.size() > 0 ){  
      Element child = myDom.createElement( "Attributes" );
      myXML.appendChild( child );
      this.createXMLHelp( attributes, child );
    }
  }  
  
  
  /**
   * @return Palauttaa oliolle annetun loogisen nimen
  */
  public String getModelName(){
    return this.name;
  }

  /**
   * @return Palauttaa kontekstimallin sisltmt attribuutit taulukkona
  */
  public ContextModelAttributeInterface[] getAllAttributes(){
    int index = 0;
    ContextModelAttributeInterface[] toReturn = 
          new ContextModelAttributeInterface[ forcedAttributes.size() + attributes.size() ];

    for( int i=0; i<forcedAttributes.size(); i++ ){
      toReturn[index] = ( (AttributeWeightPair)forcedAttributes.get(i) ).getAttribute();
      index++;
    }
    for( int i=0; i<attributes.size(); i++ ){
      toReturn[index] = ( (AttributeWeightPair)attributes.get(i) ).getAttribute();
      index++;
    }
    return toReturn;
  }
  
  /**
   * @param attr Attribuutti-ilmentym, jonka tilaa kysytn
   * @return Palauttaa totuusarvon siit, lytyyk annettu attribuutti-ilmentym
   * pakotetuiksi mritellyist attribuuteista.
  */ 
  public boolean isForcedAttribute(ContextModelAttributeInterface attr) {
    for( int i=0; i<forcedAttributes.size(); i++ ){
      if (((AttributeWeightPair)forcedAttributes.get(i)).getAttribute() == attr)
       return true;
    }
    return false;
  }

  /**
   * @param attr Attribuutti-ilmentym, jonka painoarvo halutaan
   * @return Palauttaa kokonaislukuarvon, joka kuvaa annetuun attribuutti-ilmentymn
   * liitetyn painoarvon. Jos ilmentym ei lydy palautetaan 0
  */   
  public int getAttributeWeight(ContextModelAttributeInterface attr) {
    for( int i=0; i<forcedAttributes.size(); i++ ){
      AttributeWeightPair a = (AttributeWeightPair)forcedAttributes.get(i);
      if (a.getAttribute() == attr)
        return (int)a.getWeight();
    }
    for( int i=0; i<attributes.size(); i++ ){
      AttributeWeightPair a = (AttributeWeightPair)attributes.get(i);
      if (a.getAttribute() == attr)
        return (int)a.getWeight();
    }
    return 0;
  }
  
//*****************************************************************************
//*****************************************************************************
//
//**************        PROTECTED METODIT         *****************************
//
//*****************************************************************************  

  
  
  
  /**
   * Laskee kontekstimallille arvon ja liitt itsens attribuutiksi annettuun Message-olioon
   * @param msg
  */
  protected void startProcessing( Message msg ){
    // Malliin asetettujen attribuuttinen painoarvojen summa
    long totalSum = 0;
    long modelResult = 0;
    long currentWeight = 0;
    ContextModelAttributeInterface currentAttr = null;
    
    for( int i=0; i<forcedAttributes.size(); i++ ){
      currentWeight = ( (AttributeWeightPair)forcedAttributes.get(i) ).getWeight();
      totalSum += currentWeight;
      currentAttr = ( (AttributeWeightPair)forcedAttributes.get(i) ).getAttribute();
      //Suoritetaan vertailu pakotetuille attribuuteille
      if ( currentAttr.compare( msg ) )
        modelResult += currentWeight;
      else{
        //Jos yksikin oli virheellinen malli saa arvon 0
        ContextModelValueAttribute attr = new ContextModelValueAttribute();
        attr.setValue( new Integer(0) );
        attr.setName( this.name );
        msg.addAttribute( attr );
        return;
      }
    }

    for( int i=0; i<attributes.size(); i++ ){
      currentWeight = ( (AttributeWeightPair)attributes.get(i) ).getWeight();
      totalSum += currentWeight;
      currentAttr = ( (AttributeWeightPair)attributes.get(i) ).getAttribute();
      if ( currentAttr.compare( msg ) ) {
        modelResult += currentWeight;
      }
    }
    ContextModelValueAttribute attr = new ContextModelValueAttribute();
    
    if ( totalSum == 0 )
      attr.setValue(  new Integer(0) );
    else
      attr.setValue(  new Integer( (int)( 100*((1.0*modelResult)/totalSum) )  ) );
    attr.setName( this.name );
    msg.addAttribute( attr );
  }
  
  
  
  
  
//*****************************************************************************
//*****************************************************************************
//
//**************        PRIVATE METODIT         *******************************
//
//*****************************************************************************  

  private void createXMLHelp( ArrayList list, Element parent ){
    Element item = null;
    ContextModelAttributeInterface attr = null;
    Document dom = parent.getOwnerDocument();
    
    for( int i=0; i < list.size(); i++ ){
        item = dom.createElement("Item");
        String w = "" + ( (AttributeWeightPair)(list.get(i)) ).getWeight();
        item.setAttribute("Weight", w ); 
        attr = ( (AttributeWeightPair)(list.get(i)) ).getAttribute();
        item.setAttribute("Name", attr.getName() );
        parent.appendChild( item );
        attr.createXML( item );
      }
  }




  
  //
  // Luo NodeList:sta lytyvt ContextModelAttributeInterfacen toteuttavat attribuutit
  // ja lis ne annettuun ArrayList:iin
  //
  private boolean loadAttributes( NodeList itemList, ArrayList list ) {
    boolean success = true;
    for( int i=0; i < itemList.getLength(); i++ ){
      Element item = (Element)itemList.item(i);
      long w = Long.parseLong( item.getAttribute("Weight") );
      String name = item.getAttribute("Name");
      ContextModelAttributeInterface attr = AttributeLoader.newContextModelAttributeInstance( name );
      if ( attr != null ){
        success = attr.loadXML(item.getFirstChild());
        if( !success ){
          Log.output("ContextModel").println("Attribuutin " + name + " XML lataus ei onnistunut");
          break;
        }
      }else{
        Log.output("ContextModel").println("AttributeLoader palautti null - attribute : " + name);
        success = false;
        break;
      }
      AttributeWeightPair awPair = new AttributeWeightPair( attr, w );
      list.add( awPair );
    }
    return success;
  }


  
//*****************************************************************************
//*****************************************************************************
//
//**************        SISLUOKKA        *************************************
//
//*****************************************************************************  


  //
  // Sisluokka attribuutti ja painoarvo parien hallintaan
  //  
  class AttributeWeightPair{

    private ContextModelAttributeInterface data;
    private long weight;

    private AttributeWeightPair( ContextModelAttributeInterface ob, long value ){
      this.data = ob;
      this.weight = value;
    }

    public long getWeight(){
      return this.weight;
    }

    public ContextModelAttributeInterface getAttribute(){
      return this.data;
    }
  }
  
  
  
  //DEBUG************************************************************************
  public static void main(String[] args){
    AttributeLoader.loadClasses();
    Log.output("ContextModel").println("TESTI MAIN : Luodaan uusi kontekstimalli : TestContextModel");
    ContextModel p = new ContextModel("TestContextModel");
    Log.output("ContextModel").println("TESTI MAIN : Listn kontekstimalliin uudet attribuutit");
    //AttributeStringArray
    String[] c = { "testi" , "testi2" };
    String[] b = { "testi" , "testi2" };
    p.addAttribute("sender","==", c, false, 100);
    p.addAttribute("sender","==", b, true, 101 );
    p.addAttribute("Subject","==",new String("Testi"), false, 102 );
    p.addAttribute("Subject","==",new String("Testi"), true, 103 );    
    
    try {
     Document dom = XMLUtil.newXMLDocument();
     Log.output("ContextModel").println("TESTI MAIN : Testi Malli luotu");
     p.createXML( dom );     
     Log.output("ContextModel").println("TESTI MAIN : Testi Malli listty dokumenttiin");
     
     Log.output("ContextModel").println("TESTI MAIN : Dokumetti luotu loppuun");
     XMLUtil.outputPrinting( dom, Log.output("ContextModel") );
     
     ContextModel a = new ContextModel( dom.getFirstChild() );
     Log.output("ContextModel").println("TESTI MAIN : Kontekstimalli luotu uudestaan");
     dom = XMLUtil.newXMLDocument();
     a.createXML( dom );
     Log.output("ContextModel").println("TESTI MAIN : Dokumetti luotu uudestaan");
     XMLUtil.outputPrinting( dom, Log.output("ContextModel2"));
     
    }catch(Exception e){
      Log.output("ContextModel").println("ERROR, see err.log"); 
      Log.stack(e);
    }
  }
  //DEBUG************************************************************************

}
