/*
Tiedosto: DatabaseProvider.java
Luokat:   DatabaseProvider

Tekij:   Olli Pettay
Luonti:   17.11.2002

Muutokset:
  17.11.2002  Luokan luonti
  02.12.2002  Olli Pettay
              Yhdistminen Xindiceen ja
              metodien muutoksia.

*/


package converge.kernel;

import org.w3c.dom.*;
import java.io.*;
//import java.util.*;
import org.apache.xindice.server.*;
import org.apache.xindice.core.*;
import org.apache.xindice.core.data.*;
import org.apache.xindice.server.services.*;
import org.apache.xindice.util.*; 
import org.apache.xindice.xml.dom.DOMParser;
import org.apache.xindice.xml.TextWriter;
import converge.service.Message;
/**
* Tm luokka huolehtii jrjestelmn tietokantayhteyksist.
* Tietokantana kytetn <a href="http://www.apache.org">Apache</a>n 
* <a href="http://xml.apache.org/xindice">Xindice</a> 
*/
public class DatabaseProvider {
  private static DatabaseProvider db = null;
  private Kernel kernel = null;
  
  public static final String SYSTEM_PREF   = "pref";
  public static final String USERS         = "users";
  public static final String GROUPS        = "groups";
  
  public static final String PREF          = "pref";
  public static final String PROFILES      = "profiles";
  public static final String CONTEXTMODELS = "contextmodels";
  public static final String VIEWS         = "views";
  public static final String DATASOURCES   = "datasources";
  public static final String PREFERENCE    = "preferences.xml";
  //Ei groupilla
  public static final String MESSAGES      = "messages";
  
  private static String[] defCols = {
    SYSTEM_PREF, USERS, GROUPS
  };
  
  
  private DatabaseProvider() {
         File cfg = new File("config/system.xml");
         this.kernel = new Kernel(cfg.getAbsolutePath());
         this.init();
  }
  
  /**
  * Ensimmisell kutsukerralla tm metodi kynnist tietokannan.
  * Seuraavilla kutsukerroilla palautetaan vain viite siihen.
  *
  *@return Palautetaan jrjestelmn ainoa DatabaseProvider-olio
  */
  public synchronized static DatabaseProvider getInstance() {
    if (db == null)
      db = new DatabaseProvider();
    return db;
  }
  
  /**
  *@return Palautetaan DatabaseProviderin juurikokoelma
  */
  protected synchronized Database getDatabase(){
    Service ser = this.kernel.getService("db");
    if (ser == null) {
      /*Log.err.println("No Xindice");*/
      return null;
    }
    return ((XindiceService)ser).getDatabase();
  }
  
  /**
  * Tietokannan sulkeminen. Mikli tmn jlkeen halutaan uudelleenkynnist
  * tietokanta, onnistuu se kutsumalla getInstance-metodia, joka luo uuden 
  * tietokantayhteyden.
  */
  public synchronized void shutdown(){
    if (kernel != null)
      kernel.shutDown(0, true);
    kernel = null;
  }
  
  // Tm on vain otettu konstruktorista ulos omaksi metodikseen.
  private void init() {
    for (int i = 0; i < defCols.length; ++i) {
      String name = defCols[i];
      try {
      Collection col = this.getDatabase().getCollection(name);
      if (col == null)
        this.createCollection(this.getDatabase(), name);
      //DEBUG
      else
        //System.err.println("DatabaseProvider[init]: there was : " + name);
        ;
      }
      //DEBUG
      catch(Exception e){
        Log.stack(e);
        }
    }
    
  }

  /**
  * Luodaan uusi kokoelma <i>name</i> parametrin <i>col</i> alle.
  *
  * @param col Kokoelma, johon uusi kokoelma luodaan.
  * @return Palautetaan luotu kokoelma.
  */
  protected Collection createCollection(Collection col, String name) 
                                                        throws Exception{
    String collectionConfig =
      "<collection compressed=\"true\" name=\"" + name + "\">" +
      "   <filer class=\"org.apache.xindice.core.filer.BTreeFiler\" gzip=\"true\"/>" +
      "</collection>";
      return col.createCollection(name, 
                new Configuration(DOMParser.toDocument(collectionConfig), false));
    
  }
  
  
 /////////////////////////////////////////////////////////////////////
 //              User
 /////////////////////////////////////////////////////////////////////
  /**
  * Kyttjn kokoelman (/db/users/<b>name</b>) haku tietokannasta
  *
  *@param name Kyttjn nimi
  *@return Palautetaan saatu kokoelma tai null.
  */
  protected synchronized Collection getUserCollection(String name) {
    /*if (name.startsWith("group:"))
      return getGroupCollection(name);*/
    try {
      Database db = this.getDatabase();
      Collection users = null;
      users = db.getCollection(USERS);
      return users.getCollection(name);
    }catch(Exception e) {System.err.println("Error " + e);}
    return null;
  }
  
  /**
  * Uuden kyttjkokoelman luonti.
  * Tm yhteydess luodaan mys alikokoelmat pref, profiles, contextmodels, views
  * ja messages.
  *
  *@param name Kyttjn nimi.
  *@return Palautetaan luotu kokoelma tai null.
  */
  protected synchronized Collection newUserCollection(String name) {
    /*if (name.startsWith("group:"))
      return newGroupCollection(name);*/
    //Tarkastetaan ettei kyttjll ole jo kokoelmaa.
    if (this.getUserCollection(name) != null) {
      Log.err.println("ERROR: trying to redeclare user collection!");
      return null;
    }
    try {
      Collection uc =  this.createCollection(this.getDatabase().getCollection(USERS), name);
      this.createCollection(uc, PREF);
      this.createCollection(uc, PROFILES);
      this.createCollection(uc, CONTEXTMODELS);
      this.createCollection(uc, DATASOURCES);
      this.createCollection(uc, VIEWS);
      this.createCollection(uc, MESSAGES);
      return uc;
    }catch(Exception e) {System.err.println("Error " + e);}
    return null;
  }
  
  /**
  *Poistetaan kyttjn kokoelma tietokannasta.
  *
  *@param name Kyttjn nimi.
  */
  protected synchronized void removeUserCollection(String name) {
   /*if (name.startsWith("group:")) {
      removeGroupCollection(name);
      return;
   }*/
    try {
      Database db = this.getDatabase();
      Collection users = null;
      users = db.getCollection(USERS);
      users.getCollection(name).dropCollection(users.getCollection(name));
    }catch(Exception e) {System.err.println("Error " + e);}
  }
 

 ////////////////////////////////////////////////////////////
 //   Nm toimivat sek userille ett groupille. Jos subcollection on null, 
 //                                                   se jtetn huomiotta
 ///////////////////////////////////////////////////////////
 
 /**
 * Listn dokumentti kokoelmaan tai sen alikokoelmaan.
 *
 *@param collection Kokoelma, johon lisys kohdistuu
 *@param subcollection Mikli tm ei ole null, listn dokumentti kokoelmaan <i>collection/subcollection/</i>
 *@param id Listtvn dokumentin tunniste
 *@param dom Listtv XML-dokumentti
 */
  protected synchronized void setDocument(Collection collection,
                            String subcollection, String id, Document dom) {
    try {
    Collection mc = collection;
    if (subcollection != null)
       mc = collection.getCollection(subcollection);
    //this.deleteDocument(collection, subcollection, id); //poistetaan vanha
    mc.setDocument(id, dom);
    }catch(Exception e){System.err.println("Error " + e);}
  }
  
  /**
  * Dokumentin haku kokoelmasta.
  *
  *@param collection Kokoelma, johon haku kohdistuu
  *@param subcollection Mikli tm ei ole null, haetaan dokumenttia kokoelmasta <i>collection/subcollection/</i>
  *@param id Dokumentin tunnniste.
  */
  protected synchronized Document getDocument(Collection collection,
                                          String subcollection, String id) {
    try {
      Collection mc = collection;
      if (subcollection != null)
        mc = collection.getCollection(subcollection);
      return mc.getDocument(id);
    } catch (Exception e) {System.err.println("Error " + e);}
    return null;
  }
  /**
  * Dokumentin poisto kokoelmasta.
  *
  *@param collection Kokoelma, johon poisto kohdistuu
  *@param subcollection Mikli tm ei ole null, poistetaan dokumentti kokoelmasta <i>collection/subcollection/</i>
  *@param id Dokumentin tunnniste.
  */
  protected synchronized void deleteDocument(Collection collection, 
                                            String subcollection,String id) {
    try {
      Collection mc = collection;
      if (subcollection != null)
        mc = collection.getCollection(subcollection);
      mc.remove(id);
    } catch (Exception e) {;}
    return;
  }

  
  /**
  *Kokoelman tyhjennys.
  *
  *
  *@param parent Kokoelman vanhempi
  *@param collection Kokoelman nimi
  *@deprecated Tm ei ole turvallinen ratkaisu
  */
  protected synchronized boolean setCollectionEmpty(Collection parent, String collection) {
    boolean ok = false;
    try {
      Collection col = parent.getCollection(collection);
      if (col != null) { 
        ok = parent.dropCollection(col);
        this.createCollection(parent, collection);
      }
    } catch (Exception e) {
      Log.stack(e);
    }
    return ok;
  }
 /////////////////////////////////////////////////////////////////////
 //              Group
 /////////////////////////////////////////////////////////////////////
 
  /**
  * Ryhmn kokoelman (/db/groups/<b>name</b>) haku tietokannasta
  *
  *@param name Ryhmn nimi
  *@return Palautetaan saatu kokoelma tai null.
  */
 protected synchronized Collection getGroupCollection(String name) {
    try {
      Database db = this.getDatabase();
      Collection groups = null;
      groups = db.getCollection(GROUPS);
      return groups.getCollection(name);
    }catch(Exception e) {System.err.println("Error " + e);}
    return null;
  }
  
  /**
  * Uuden ryhmkokoelman luonti.
  * Tm yhteydess luodaan mys alikokoelmat pref, profiles ja contextmodels.
  * 
  *
  *@param name Ryhmn nimi.
  *@return Palautetaan luotu kokoelma tai null.
  */
  protected synchronized Collection newGroupCollection(String name) {
    //Tarkastetaan ettei ryhmll ole jo kokoelmaa.
    if (this.getGroupCollection(name) != null) {
      System.err.println("ERROR: trying to redeclare group collection!");
      return null;
    }
    try {
      Collection gc = this.createCollection(this.getDatabase().getCollection(GROUPS), name);
      this.createCollection(gc, PREF);
      this.createCollection(gc, PROFILES);
      this.createCollection(gc, DATASOURCES);
      this.createCollection(gc, CONTEXTMODELS);
      //this.createCollection(gc, VIEWS);
      return gc;
    }catch(Exception e) {System.err.println("Error " + e);}
    return null;
  }
  
  /**
  *Poistetaan ryhmn kokoelma tietokannasta.
  *
  *@param name Ryhmn nimi.
  */  
  protected synchronized void removeGroupCollection(String name) {
    try {
      Database db = this.getDatabase();
      Collection groups = null;
      groups = db.getCollection(GROUPS);
      groups.getCollection(name).dropCollection(groups.getCollection(name));
    }catch(Exception e) {System.err.println("Error " + e);}
  }
  
  
  

 
 
 
 ///////////////////////////////////////////////
 // DatabaseProviderin suunnittelua ja testausta varten
 ///////////////////////////////////////////////
 /**
 *For testing only!
 *
 */
  public static void main(String[] args) {
    String testid = "id12345";
    String testview = "INBOX";
    String testview2 = "WORK";
    String username = "db_tester";
    ViewManager vm = null;
    System.err.println("TEST: DatabaseProvider");
    DatabaseProvider adb = getInstance();
    Database db = adb.getDatabase();
    String[] cols = null;
    if (db != null && db instanceof Collection) {
      System.err.println("Found a database");
      try {
       cols = db.listCollections();
      for (int i = 0; cols != null && i < cols.length; ++i)
        System.err.println("A Collection: " + cols[i]);
      Collection c = db.getCollection("system");
      if (c != null) {
        cols = c.listCollections();
        for (int i = 0; cols != null && i < cols.length; ++i)
          System.err.println("A SystemCollection: " + cols[i]);
      }
      Collection uc;
      if ((uc = adb.getUserCollection(username)) == null) {
         uc = adb.newUserCollection(username);
         adb.setDocument(uc, MESSAGES,testid,
              DOMParser.toDocument(new FileInputStream(new File("nonsense/test.xml"))));
      }
      
      
      
      Collection acol = adb.getUserCollection(username);
      acol = acol.getCollection(MESSAGES);
      String[] msgs = acol.listDocuments();
      for (int i=0; i < msgs.length; ++i)
        System.out.println(msgs[i]);
      /*else {
        Collection uc = adb.getUserCollection("db_tester");
        Document d = adb.getDocument(uc, MESSAGES ,"id12345");
        TextWriter tw = new TextWriter(d);
        tw.write(System.out);
        adb.removeUserCollection("db_tester");
      }*/
      
      User user = new User(username, username);
      vm = ViewManager.getInstance();
      
      
      ViewMonitor.setMessageToView(user, testid, testview);
      
      
      ViewMonitor.setMessageToView(user, testid, testview2);
      
      Document dom = ViewMonitor.getMessage(user, testid);
      if (dom != null) {
        TextWriter tw = new TextWriter(dom);
        tw.write(System.out);
      }
      
      //Nkymien lpikynti
      String[] views = ViewMonitor.getViewNames(user);
      System.err.println("Views");
      for (int i = 0; views != null & i < views.length; ++i) {
        System.err.println(views[i] + ":");
        View v = ViewMonitor.getView(user,views[i]);
        if (v != null ){
          String[] s = v.getMessageIDs();
          Header[] h = v.getMessageHeaders();
          for (int k = 0; k < s.length; ++k) {
            System.err.println(s[k] + ":");
            System.err.println("Subject:" + h[k].getSubject());
            System.err.println("Date:" + h[k].getDate());
            System.err.println("Sender:" + h[k].getSender());
          }
        }
      }
      
      ViewMonitor.deleteMessageFromView(user, testid ,testview);
      System.err.println("Should be in " + testview2);
      
      
      //vm.shutdown();
      //Nkymien lpikynti
      views = ViewMonitor.getViewNames(user);
      System.err.println("Views");
      for (int i = 0; views != null & i < views.length; ++i) {
        System.err.println(views[i] + ":");
        View v = ViewMonitor.getView(user,views[i]);
        if (v != null ){
          String[] s = v.getMessageIDs();
          Header[] h = v.getMessageHeaders();
          for (int k = 0; k < s.length; ++k) {
            System.err.println(s[k] + ":");
            System.err.println("Subject:" + h[k].getSubject());
            System.err.println("Date:" + h[k].getDate());
            System.err.println("Sender:" + h[k].getSender());
          }
        }
      }
      
      ViewMonitor.deleteMessage(user, testid);
      
      
      //Nkymien lpikynti
      views = ViewMonitor.getViewNames(user);
      System.err.println("Views");
      for (int i = 0; views != null & i < views.length; ++i) {
        System.err.println(views[i] + ":");
        View v = ViewMonitor.getView(user,views[i]);
        if (v != null ){
          String[] s = v.getMessageIDs();
          Header[] h = v.getMessageHeaders();
          for (int k = 0; k < s.length; ++k) {
            System.err.println(s[k] + ":");
            System.err.println("Subject:" + h[k].getSubject());
            System.err.println("Date:" + h[k].getDate());
            System.err.println("Sender:" + h[k].getSender());
          }
        }
      }
      
      ViewMonitor.deleteView(user, testview);
      
      //Nkymien lpikynti
      views = ViewMonitor.getViewNames(user);
      System.err.println("Views");
      for (int i = 0; views != null & i < views.length; ++i) {
        System.err.println(views[i] + ":");
        View v = ViewMonitor.getView(user,views[i]);
        if (v != null ){
          String[] s = v.getMessageIDs();
          Header[] h = v.getMessageHeaders();
          for (int k = 0; k < s.length; ++k) {
            System.err.println(s[k] + ":");
            System.err.println("Subject:" + h[k].getSubject());
            System.err.println("Date:" + h[k].getDate());
            System.err.println("Sender:" + h[k].getSender());
          }
        }
      }
      
      
      }catch(Exception e){System.err.println("Error " + e);}
    }
    //MessageRules.main(args);
    //adb.shutdown();
    
    /*RuleHandler rh = new RuleHandler();
    try{
      rh.test(DOMParser.toDocument(new FileInputStream(new File("converge/kernel/MessageRules.xml"))));
    }catch(Exception e){System.err.println(e);}
   */
  }
 
}
