/*
Tiedosto: Rule.java
Luokat:   Rule

Tekij:   Olli Pettay
Luonti:   23.11.2002

Muutokset:
  23.11.2002  Luokan luonti
  05.12.2002  Olli Pettay
              toteutusta
              
TODO: Tarkista ettei sytteiss ole ]]> -merkint!!!

*/


package converge.kernel;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xindice.xml.TextWriter;
/**
*Luokka sntjen ksittelyyn.
*/
public class Rule implements XMLContentInterface{
  public static final int EVENT_ADD = 1;
  public static final int EVENT_DELETE = 2;

  private String name = null;
  private LinkedList conditions;
  private LinkedList actions;
  private boolean opAdded = false;
  
  public Rule(String name) {
    this.name = name;
    this.conditions = new LinkedList();
    this.actions = new LinkedList();
    this.opAdded = true;
  }
  
  protected Rule(){
    this.conditions = new LinkedList();
    this.actions = new LinkedList();
    this.opAdded = true;
  }
  
  
  /**
  *Palautetaan snnn nimi.
  *
  */
  public String getName() {
    return this.name;
  }
  
  /**
  *Palauttaa listan annetuista ehdoista.
  *Lista sislt Triple ja ConditionalOperator -olioita
  *siin jrjestyksess miss addCondition (=Triple) ja
  *nextCondition (=ConditionalOperator) -metodeja on kutsuttu.
  */
  public List getConditions() {
    return this.conditions;
  }
  
  
  //ehto, voi olla useita, jos vliss kutsutaan nextCondition-metodia
  /**
  *Listn uusi sntn ehto.
  *
  *@param attributeName viitattavan attribuutin nimi
  *@param value arvo
  *@param expressionType vertailutapa
  */
  public void addCondition(String attributeName, String value, String expressionType) {
    if (!this.opAdded) return; //Ei voida list
    attributeName = removeQuot(attributeName);
    value = removeQuot(value);
    expressionType = removeQuot(expressionType);
    conditions.add(new Triple(attributeName, value, expressionType));
    this.opAdded = false;
  }
  
  
  //Parametrin AND OR
  /**
  *Mikli halutaan list useita ehtoja tytyy niiden vlill kutsua
  *tt metodia.
  *
  *@param conditionalOperator OR tai AND kertoo ehtojen yhdistmistavan
  */
  public void nextCondition(String conditionalOperator) {
    if (this.opAdded) return; //Ei voida laittaa kahta op:ta 
    conditionalOperator = removeQuot(conditionalOperator);
    this.conditions.add(new ConditionalOperator(conditionalOperator));
    this.opAdded = true;
  }
  
  
  /**
  *Palauttaa listan toiminnoista.
  *Lista sislt Consequence-olioita.
  */
  public List getConsequences() {
    return this.actions;
  }
  
  //tapahtuma, voidaan list useita
  /**
  *Listn uusi toiminto sntn.
  *Tm metodia voi kutsua useasti, ja kaikki saadut toiminnot liitetn 
  *sntn.
  *Tyypin ollessa Rule.EVENT_ADD, listn sntjenksittelyn yhteydess
  *syntyvaan ActionEvent-olioon uusi arvo parametrien mukaisesti.
  *Mikli tyyppi on Rule.EVENT_DELETE poistetaan nimen mukainen 
  *toiminto, tai mikli on annettu mys arvo, vain nimeen liitetty tietty arvo.
  *
  *@param eventType joko Rule.EVENT_ADD tai Rule.EVENT_DELETE 
  *@param eventname toiminnon nimi
  *@param value toiminnolle annettava arvo, voi olla null, mikli eventType 
  *                                                   on Rule.DELETE
  */
  public void consequence(int eventType, String eventname, String value) {
    eventname = removeQuot(eventname);
    value = removeQuot(value);
    actions.add(new Consequence(eventType, eventname, value));
  }
  
  /**
  *Palautetaan Declaration-taulukko, joka sislt viitteen sntjen
  *ksittelyss kytettvien olioiden nimiin ja tyyppeihin.
  *
  */
  public org.drools.rule.Declaration[] getParametersForDrools(){
    try {
    org.drools.rule.Declaration[] ret = new org.drools.rule.Declaration[2];
    String name1 = "message";
    org.drools.semantics.java.ClassObjectType type1 = 
         new org.drools.semantics.java.ClassObjectType(Class.forName("converge.service.Message"));
    org.drools.rule.Declaration first = new org.drools.rule.Declaration(type1, name1);
    ret[0] = first;
    
    String name2 = "event";
    org.drools.semantics.java.ClassObjectType type2 = 
         new org.drools.semantics.java.ClassObjectType(Class.forName("converge.kernel.ActionEvent"));
    
    org.drools.rule.Declaration second = new org.drools.rule.Declaration(type2, name2);
    ret[1] = second;
    return ret;
    }catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }
  
  /**
  *Palautetaan Droolsin ymmrtmss muodossa snnn ehto
  */
  public org.drools.semantics.java.ExprCondition getConditionForDrools() {
    try {
    return new org.drools.semantics.java.ExprCondition(
          this.makeConditionCode("message", "getMessageAttribute"),
          this.getParametersForDrools());
    }catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }
  /**
  *Palautetaan Droolsin ymmrtmss muodossa snnn toiminto.
  */
  public org.drools.semantics.java.BlockConsequence getConsequenceForDrools() {
    return new org.drools.semantics.java.BlockConsequence(this.makeConsequenceCode());
  }
  
  /**
  *Luodaan XML.
  */
  public void createXML(Node par){
    Element parent = (Element)par;
    Document dom = parent.getOwnerDocument();
    Element el = dom.createElement("rule");
    el.setAttribute("name", this.name);
    
    Element cond = dom.createElement("conditions");
    Iterator it = conditions.iterator();
    while(it.hasNext()) {
      Object next = it.next();
      if (next instanceof ConditionalOperator && it.hasNext())
        ((ConditionalOperator)next).createXML(cond);
      else if (next instanceof Triple)
        ((Triple)next).createXML(cond);
    }
    el.appendChild(cond);
    
    Element conq = dom.createElement("consequences");
    it = actions.iterator();
    while(it.hasNext())
      ((Consequence)it.next()).createXML(conq);
    el.appendChild(conq);
    
    
    parent.appendChild(el);
  }
  
  
  /**
  *Ladataan XML.
  */
  public boolean loadXML(Node cont){
    Element content = (Element)cont;
    Document dom = cont.getOwnerDocument();
    if (!content.getTagName().equals("rule"))
      return false;
    this.name = content.getAttribute("name");
    Element child = (Element)content.getFirstChild();
    while (child != null) {
      if (child.getTagName().equals("conditions")) {
        Element subchild = (Element)child.getFirstChild();
        while (subchild != null) {
          Triple t = new Triple();
          ConditionalOperator c = new ConditionalOperator();
          if (t.loadXML(subchild)) {
            this.opAdded = false;
            this.conditions.add(t);
          }
          else if (c.loadXML(subchild)) {
            this.opAdded = true;
            this.conditions.add(c);
          }
          subchild = (Element)subchild.getNextSibling();
        }
      }
      else if (child.getTagName().equals("consequences")) {
        Element subchild = (Element)child.getFirstChild();
        while(subchild != null) {
          Consequence c = new Consequence();
          if (c.loadXML(subchild))
            this.actions.add(c);
          subchild = (Element)subchild.getNextSibling();
        }
      }
      child = (Element)child.getNextSibling();
    }
    return true;
  }
  
  
  /**
  *Luodaan ehdoista Java-koodia.
  */
  protected String makeConditionCode(String objectname, String methodname){
    StringBuffer code = new StringBuffer("");
    Iterator it = conditions.iterator();
    if (!it.hasNext())
      code.append("true");
    while(it.hasNext()) {
      Object next = it.next();
      if (next instanceof ConditionalOperator && it.hasNext())
        code.append(" " + ((ConditionalOperator)next).getOp() + " ");
      else if (next instanceof Triple)
        code.append(((Triple)next).makeCode(objectname, methodname));
    }
    return code.toString();
  }
  
  /**
  *Luodaan toiminnoista Java-koodia.
  */
  protected String makeConsequenceCode(){
    StringBuffer code = new StringBuffer("");
    Iterator it = actions.iterator();
    if (!it.hasNext())
      code.append(";");
    while(it.hasNext())
      code.append(((Consequence)it.next()).makeCode());
    return code.toString();
  }
  
  
  /*
  *Testi.
  *
  */
  /*public static void main(String[] args){
    Profile p = new Profile(new User("name", "pw"), "TestProfile");
    Rule r = new Rule("TestRule");
    r.addCondition("sender", "pekka.puupaa@helsinki.fi", "=");
    r.nextCondition("OR");
    r.addCondition("daytime", "12:30", "<");
    r.nextCondition("and");
    r.addCondition("day", "friday", "=");
    
    r.consequence(Rule.EVENT_ADD, "settoview", "INBOX");
    r.consequence(Rule.EVENT_DELETE, "settoview", null);
    r.consequence(Rule.EVENT_DELETE, "settoview", "INBOX");
    p.setRule(r);
    System.out.println(r.makeConditionCode("message", "getAttribute"));
    System.out.println(r.makeConsequenceCode());
    try {Set
     Document dom = XMLUtil.newDroolsDocument();
     NodeList nl = dom.getElementsByTagNameNS(XMLUtil.NS_DROOLS, "rule-set");
     r.createXMLForDrools(nl.item(0));
     TextWriter tw = new TextWriter(dom);
     tw.write(System.out);
     System.out.println("");
     
     dom = XMLUtil.newXMLDocument();
     Element el = dom.createElement("rules");
     dom.appendChild(el);
     r.createXML(el);
     tw = new TextWriter(dom);
     tw.write(System.out);
     System.out.println("");
     
     
     Rule new_r = new Rule();
     new_r.loadXML(dom.getDocumentElement().getFirstChild());
     dom = XMLUtil.newDroolsDocument();
     nl = dom.getElementsByTagNameNS(XMLUtil.NS_DROOLS, "rule-set");
     new_r.createXMLForDrools(nl.item(0));
     tw = new TextWriter(dom);
     tw.write(System.out);
     System.out.println("");


     System.out.println("TestScheduledRules");
     ScheduledRule sr = new ScheduledRule("A_Sched_Rule");
     sr.addCondition("place", "helsinki", "=");
     sr.nextCondition("OR");
     sr.addCondition("daytime", "12:30", "<");
     sr.nextCondition("and");
     sr.addCondition("day", "friday", "=");
     sr.consequence(Rule.EVENT_ADD, "get", "imap.cs.helsinki.fi");
     dom = XMLUtil.newDroolsDocument();
     nl = dom.getElementsByTagNameNS(XMLUtil.NS_DROOLS, "rule-set");
     sr.createXMLForDrools(nl.item(0));
     tw = new TextWriter(dom);
     tw.write(System.out);
     System.out.println("");


     Scheduler sc = new Scheduler("", new Date());
     sc.setRule(sr);
     p.addScheduledRule(sc);
     
     p.setContextModelValue("WORK", 70);
     
     System.out.println("\n\n[Profile1: " + p.getName() + "]\n\n");
     dom = XMLUtil.newXMLDocument();
     p.createXML(dom);
     tw = new TextWriter(dom);
     tw.write(System.out);
     System.out.println("");
     
     Profile p2 = new Profile(new User("", ""));
     p2.loadXML(dom.getDocumentElement());
     
     System.out.println("\n\n[Profile2: " + p2.getName() + "]\n\n");
     dom = XMLUtil.newXMLDocument();
     p2.createXML(dom);
     NodeList gl = dom.getElementsByTagName("rule");
     System.err.println("\ncount\n" + gl.getLength());
     tw = new TextWriter(dom);
     tw.write(Log.output("kernel"));
     System.out.println("");
     
     System.out.println("for Drools...");
     tw = new TextWriter(p2.getRuleSet());
     tw.write(System.out);
     System.out.println("");
     System.out.println("XMLUtil.outputPrinting:");
     XMLUtil.outputPrinting(p2.getRuleSet(), null);
     

     
    }catch(Exception e){System.err.println("ERROR "); e.printStackTrace();}
  }
  */
  
  /**
  *Poistetaan " ja ' -merkit merkkijonosta.
  */
  private static String removeQuot(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("\"", "");
    return in;
  }
  
}

