/*
Tiedosto: Scheduler.java
Luokat:  Scheduler

Tekij:   Olli Pettay
Luonti:   19.11.2002


Muutokset:
  19.11.2002 Olli Pettay  
              Luokan luonti

*/


package converge.kernel;
import org.w3c.dom.*;
import java.util.*;

/**
*Ajastin
*/
public class Scheduler implements XMLContentInterface{
  public static final int MINUTE = 1;
  public static final int HOUR = 2;
  public static final int DAY = 3;
  public static final int WEEK = 4;
  public static final int MONTH = 5;
  
  private Date date = null;
  private int type = 0;
  private int multiply =0;
  private boolean done = false;
  private Profile profile;
  private LinkedHashMap rules;
  private long lastTime = 0;
  private String name;
  
  /**
  *@param name ajastimen nimi
  *@param type jokin Schedulerin mrittmist final-tyyppisist kentist
  *@param multiply kerroin
  */
  public Scheduler(String name, int type, int multiply) {
    this.type = type;
    this.multiply = multiply;
    this.rules = new LinkedHashMap();
    this.name = name;
  }
  
  /**
  *@param name ajastimen nimi
  *@param date Mritelln tarkka pivmr ja aika
  */
  public Scheduler(String name, Date date) {
    this.date = date;
    this.rules = new LinkedHashMap();
    this.name = name;
  }
  
  public Scheduler(){
    this.rules = new LinkedHashMap();
  }
  
  /**
  *Milloin ajastin on edellisen kerran lauennut
  */
  protected long getLastTime() {
    return this.lastTime;
  }
  
  /**
  *Asetetaan tieto edellisest laukeamisajankohdasta.
  */
  protected void setLastTime(long time) {
    this.lastTime = time;
  }
  
  /**
  *Palautetaan ajastimen nimi.
  */
  public String getName() {
    return this.name;
  }
  
  /**
  *Onko ajastus konstruoitu Date-oliolla?
  */
  public boolean exact() {
    if(this.date != null) 
      return true;
    return false;
  }
  
  /**
  *Palautetaan tarkka laukeamisajankohta Date-oliona.
  */
  public Date getDate(){
    return this.date;
  }
  
  /**
  *Tyyppi.
  *Joko MINUTE, HOUR, DAY, WEEK tai MONTH.
  */
  public int getType() {
    return this.type;
  }
  
  /**
  *Kerroin.
  */
  public int getMultiply() {
    return this.multiply;
  }
  
  
  /**
  *Laukeamisajankohtien vli
  */
  public long getInterval() {
    long base = 1000;
    switch (this.type) {
      case Scheduler.MONTH: //30 vrk 
        base = base * 30;
      case Scheduler.WEEK:
        base = base * 7;
      case Scheduler.DAY:
        base = base * 24;
      case Scheduler.HOUR:
        base = base * 60;
      case Scheduler.MINUTE:
        base = base * 60;
      break;
      default:
        base = 1000000000;
        break;
    }
    base = base * this.multiply;
    System.out.println("Scheduler interval " + base);
    return base;
  }
  
  /**
  *Asetetaan snt ajastimeen.
  *
  *@param rule snt
  */
  public void setRule(ScheduledRule rule) {
    this.rules.put(rule.getName(), rule);
  }
  
  /**
  *Palautetaan snt nimen perusteella.
  *
  *@param name halutun snnn nimi
  */
  public ScheduledRule getRule(String name) {
    return (ScheduledRule)this.rules.get(name);
  }
  
  
  /**
  *Poistetaan snt ajastimesta.
  *
  *@param name poistettavan snnn nimi
  */
  public void deleteRule(String name) {
    this.rules.remove(name);
  }
  
  
  /**
  *Palautetaan ajastimeen liitettyjen sntjen nimet.
  */
  public String[] getRuleNames() {
    return (String[]) this.rules.keySet().toArray(new String[0]);
  }
  
  
  public Rule[] getRules() {
      return (Rule[])this.rules.values().toArray(new Rule[this.rules.size()]);
  }
  /**
  *Luodaan XML
  */
  public void createXML(Node par){
    Element parent = (Element) par;
    Document dom = parent.getOwnerDocument();
    Element el = dom.createElement("scheduler");
    el.setAttribute("name", this.name);
    if (this.exact())
      el.setAttribute("date", new Long(this.date.getTime()).toString());
    else {
      el.setAttribute("type", new Integer(this.type).toString());
      el.setAttribute("multiply", new Integer(this.multiply).toString());
    }
    
    Element el2 = dom.createElement("rule-set");
    Set rs = this.rules.keySet();
    Iterator it = rs.iterator();
    while (it.hasNext()) {
      ScheduledRule r = (ScheduledRule) this.rules.get(it.next());
      r.createXML(el2);
     }
    el.appendChild(el2);
    parent.appendChild(el);
  }
  
  /**
  
  */
  public boolean loadXML(Node cont){
    Element content = (Element)cont;
    Document dom = cont.getOwnerDocument();
    if (!content.getTagName().equals("scheduler"))
      return false;
    this.name = content.getAttribute("name");
    if (content.hasAttribute("date"))
      this.date = new Date(new Long(content.getAttribute("date")).longValue());
    else {
      this.type = Integer.parseInt(content.getAttribute("type"));
      this.multiply = Integer.parseInt(content.getAttribute("multiply"));
    }
    Element rs = (Element)content.getFirstChild();
    if (rs != null && rs.getTagName().equals("rule-set")) {
      Element child = (Element)rs.getFirstChild();
      while (child != null){
        ScheduledRule r = new ScheduledRule();
        r.loadXML(child);
        this.setRule(r);
        child = (Element) child.getNextSibling();
      }
    }
    return true;
  }
}
