/*
Tiedosto: UserNameCollection.java
Luokat:   UserNameCollection

Tekij:   Olli Pettay
Luonti:   19.11.2002

Muutokset:
  19.11.2002  Luokan luonti
  15.12.2002  Mikko Hiipakka 
              Rakenteen tarkistus

*/


package converge.kernel;

import org.w3c.dom.*;
import java.util.*;
/**
* Kyttjien nimien kokoelma, jota hydynnetn ryhmien ksittelyss.
*/
public class UserNameCollection implements XMLContentInterface{
  
  private HashSet users = new HashSet();;
  
  
  /**
   * Kokoelmassa silytettvn olevien kyttjien kyttjnimet
   * @return Nimet palautetaan String-taulukossa
  */ 
  public String[] getUserNames() {
    return (String[]) this.users.toArray(new String[this.users.size()]);
  }
  
  /**
   * Kyttjn lisminen kokoelmaan, jokainen liitetty kyttj
   * sisltyy kokoelmaan vain kerran
  */ 
  public void addUser(User user){
    //Ei siskkisi ryhmi
    if (user instanceof Group)
      return;
    this.users.add(user.getName());
  }
  
  /**
   * Kyttjn lisminen kokoelmaan kyttjnimen mukaan, jokainen 
   * liitetty kyttj sisltyy kokoelmaan vain kerran
  */   
  public void addUser(String user){
    this.users.add(user);
  }
  
  /**
   * Sisltk kokoelma kyttjn
   * @param name Kysyttv kyttjnimi
   * @return Totuusarvo, true, jos nimi oli kokoelmassa
  */     
  public boolean hasUser(String name){
    return this.users.contains(name);
  }
  
  /**
   * Kokoelmasta poisto kyttjnimen mukaan
  */       
  public void removeUser(String name) {
    this.users.remove(name);
  }
 

  public void createXML(Node parent) {
    Document dom = parent.getOwnerDocument();
    Element el = dom.createElement("UserNameCollection");
    parent.appendChild(el);
    Iterator it = this.users.iterator();
    while(it.hasNext()) {
      Element aUser = dom.createElement("user");
      aUser.setAttribute("name", (String)it.next());
      el.appendChild(aUser);
    }
  }

  public boolean loadXML(Node cont){
    Element content = (Element)cont;
    Document dom = cont.getOwnerDocument();
    if (!content.getTagName().equals("UserNameCollection"))
      return false;
    Element child = (Element)content.getFirstChild();
    while (child != null) {
      this.users.add(child.getAttribute("name"));
      child = (Element) child.getNextSibling();
    }
    return true;
  }
  
  //DEBUG
  public static void main(String[] args) {
    UserNameCollection g = new UserNameCollection();
    Document dom = XMLUtil.newXMLDocument();
    g.addUser(new User("TEST", ""));
    g.addUser(new User("TEST2", ""));
    g.addUser(new User("TEST3", ""));
    g.createXML(dom);
    XMLUtil.outputPrinting(dom, null);
    
    UserNameCollection g2 = new UserNameCollection();
    g2.loadXML(dom.getDocumentElement());
    dom = XMLUtil.newXMLDocument();
    g2.createXML(dom);
    XMLUtil.outputPrinting(dom, null);  
  }
  //DEBUG
}
