/*
Tiedosto: ViewManager.java
Luokat:   ViewManager, ViewManagerThread

Tekij:   Olli Pettay
Luonti:   17.11.2002


Muutokset:
  17.11.2002 Olli Pettay  
              Luokan luonti
  04.12.2002 Olli Pettay
              ViewManagerThread
              koodia

*/


package converge.kernel;
import converge.service.Message;
import java.util.*;
import org.w3c.dom.*;
import converge.attribute.*;

/**
* Tm luokka huolehtii nkymien ja viestien tietokantaksittelyst.
* TODO:testaa ViewMonitor-luokan setMessageToView-toimivuus!
*/
public class ViewManager{
  private static ViewManager vm = null;
  private ViewManagerThread  vmt = null;
  private List list = null;
  private boolean stopped;
  private boolean stop;
  
  private ViewManager(){
    this.list = Collections.synchronizedList(new LinkedList());
    this.stopped = false;
    this.stop = false;
    this.vmt = new ViewManagerThread();
  }
  /**
  *Palautetaan jrjestelmn ainoa ViewManager.
  *Tm metodin ensimmisell kutsukerralla luodaan uusi ViewManager ja
  *kynnistetn siihen liitetty sie.
  *
  */
  public static ViewManager getInstance() {
    if (vm == null) {
      vm = new ViewManager();
      vm.start();
    }
    return vm;
  }
  
  /**
  *ViewManagerin alasajo. Tm metodi j odotttamaan kunnes kaikki
  *kantaan haluttavat muutokset on tehty.
  *
  */
  public synchronized void shutdown() {
    this.stop = true;
    notifyAll();
    while (this.stopped == false)
      try{
        wait();
      }
      catch(Exception e){
        Log.stack(e);
      }
  }
  /**
  *Kynnist ViewManagerThreadin
  */
  private void start(){
    this.vmt.setViewManager(this);
    this.vmt.start();
  }
  
  /**
  *Poistetaan ksittelyodottavien listalta ensimminen ViewMonitor
  */
  public synchronized ViewMonitor removeViewMonitor() {
    while (this.list.isEmpty() && this.stop == false)
      try {
        wait();
      }
      catch(Exception e){;}
    if (this.list.isEmpty()) {
      this.stopped = true;
      notifyAll();
      return null;
    }
    
    ViewMonitor vmon = (ViewMonitor) this.list.remove(0);
    vmon.waitEvent();
    return vmon;
  }
  
  
  /**
  *Listn ksittely odottavien listalle uusi ViewMonitor.
  *Saatua oliota VOIDAAN KYTT VAIN YHDEN KERRAN jonkin
  *nkymiin liittyvn muutoksen tekemiseen.
  *Eli normaali tapa kytt ViewMonitoria on 
  *<pre>
  *  ViewMonitor vmon = ViewManager.getInstance().getNewViewMonitor();
  *  vmon.haluttuToimenpideMetodi(parmetriX, parametriY, parametriZ);
  *</pre>
  *haluttu toimenpide PIT ASETTAA HETI, kun olio on saatu getNewViewMonitor -metodilta, 
  *sill ViewManager j odottamaan toimenpiteen asettamista, eik
  *silloin muualta tulevia pyyntj ksitell. (Nin tosin vain tietyiss tapauksissa.)
  *Tmn jlkeen ViewMonitor-olio on kytetty - se on siis todellakin kertakytttavaraa!
  *
  *<b>Kytn helpottamiseksi ViewMonitor-luokka tarjoaa staattisia metodeja, jotka
  *huolehtivat itse oikeanlaisesta kutsusta.</b>
  *@see converge.kernel.ViewMonitor
  *
  *@return Palautetaan null, jos ViewManager ei ole en toiminnassa
  */
  protected synchronized ViewMonitor getNewViewMonitor() {
    if (this.stopped) { 
      notifyAll();
      return null;
    }
    ViewMonitor vmon = new ViewMonitor();
    this.list.add(vmon);
    notifyAll();
    return vmon;
  }
}


/**
* Tmn luokan ilmentym toimii ViewManagerin sikeen.
*/
class ViewManagerThread extends Thread {
  
  private ViewManager vm = null;
  private DatabaseProvider db = null;

  /**
  *Asetetaan ViewManager
  *
  *@param vm kytettv ViewManager
  */
  protected void setViewManager(ViewManager vm) {
    this.vm = vm;
  } 
  
  /**
  *Sikeen toiminnallisuus.
  */
  public void run() {
    int i = 1;
    if (this.vm == null)
      return;
    System.err.println("ViewManagerThread running...");
    this.db = DatabaseProvider.getInstance();
    while(true) {
      ViewMonitor vmon = vm.removeViewMonitor();
      if (vmon == null) //Jrjestelmn pysytys
        return;
      ViewEvent event = vmon.getEvent();
      if (event != null) { //Eventin ei pitisi olla koskaa null!!!
        //ViewEventin ksittely!!!
        User user = event.user;
        //Ryhmll ei ole nkymi!!!
        //Seuraava tarkistus pitisi olla turha
        //mutta jos joku ohjelmoi vrin.
        if (user instanceof Group) {
          //DEBUG
          System.out.println("ERROR: Trying to set message to a groupview.");
          //DEBUG
          continue;
        }
        Message message = null;
        String id = null;
        String view = null;
        
        switch(event.type) {
          case ViewEvent.SET_MESSAGE_BY_ID:
            id = event.id;
            view = event.view;
            this.setMessageToView(user, view, id);
            break;
          case ViewEvent.SET_MESSAGE:
            message = event.message;
            id = event.id;
            view = event.view;
            this.setMessage(user, message);
            this.setMessageToView(user, view, id);
            break;
          case ViewEvent.DELETE_MESSAGE_FROM_VIEW:
            id = event.id;
            view = event.view;
            this.deleteMessageFromView(user, view, id);
            break;
          case ViewEvent.DELETE_MESSAGE_TOTAL:
            id = event.id;
            this.deleteMessage(user, id);
            break;
          case ViewEvent.GET_VIEW:
            view = event.view;
            vmon.setView(this.getView(user, view));
            break;
          case ViewEvent.GET_MESSAGE_BY_ID:
            id = event.id;
            vmon.setDom(this.getMessage(user, id));
            break;
          case ViewEvent.GET_VIEW_NAMES:
            vmon.setViewNames(this.getViewNames(user));
            break;
          case ViewEvent.DELETE_VIEW:
            view = event.view;
            this.deleteView(user, view);
            break;
        }
      }
    }
  }
  
  /**
  *Listn kantaan uusi viesti
  *
  *@param user Kyttj
  *@param msg Viesti
  */
  private void setMessage(User user, Message msg){
    //DEBUG
    //System.out.println("VIEWMANAGERTHREAD: setMessage");
    //DEBUG
    Document dom = XMLUtil.newXMLDocument(); 
    msg.createXML(dom);
    this.db.setDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.MESSAGES, msg.getMessageID(), dom);

  }
  
  /**
  *Listn nkymn uusi viesti
  *
  *@param user Kyttj
  *@param view Nkym
  *@param id Viestin ID
  */
  private void setMessageToView(User user, String view, String id) {
    //Tarkastetaan, ett viesti lytyy kannasta.
    if (this.getMessage(user, id) == null) 
      return;
    Document viewdom = this.db.getDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, view);
    if (viewdom == null) {
      viewdom = XMLUtil.newXMLDocument();
      Element el = viewdom.createElement("view");
      el.setAttribute("viewname", view);
      viewdom.appendChild(el);
    }
    
    //Testataan, ettei viesti ollut jo nkymss
    if (getMessageInView(viewdom, id) == null) {
      Element el = viewdom.createElement("message");
      el.setAttribute("id", id);
      viewdom.getDocumentElement().appendChild(el);
      this.db.setDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, view, viewdom);
    }
  }
  
  //TODO: Tee viestien tuhoamisesta nopeaa!
  //Jlleen kerran: jrjettmn hidas toteutus
  /**
  *Viestin tuhoaminen nkymst. Tuhoaa sen mys kannasta mikli se toimenpiteen
  *jlkeen olisi missn nkymss.
  *
  *@param user Kyttj
  *@param view Nkym
  *@param id Viestin ID
  */
  private void deleteMessageFromView(User user, String view, String id){
    Document viewdom = this.db.getDocument(this.db.getUserCollection(user.getName()),
      DatabaseProvider.VIEWS, view);
    if (viewdom == null) return;
    Element el = this.getMessageInView(viewdom, id);
    if (el == null) return;
    viewdom.getDocumentElement().removeChild(el);
    this.db.setDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, view, viewdom);
    String[] allviews = this.getViewNames(user);
    for (int i = 0; i < allviews.length; ++i) {
      if (allviews[i].equals(view))
        continue;
      Document dom = this.db.getDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, allviews[i]);
      if (this.getMessageInView(dom, id) != null)
        return;
    }
    this.db.deleteDocument(this.db.getUserCollection(user.getName()),
        DatabaseProvider.MESSAGES, id);
    Log.output("ViewManager").println("Viesti poistettu kannasta: " + id);
    return;
  }
  
  //Aivan julmetun hidas: > O(n2), miss n = nkymt (tai nkymiss olevat viitteet)
  /**
  *Poistetaan viesti kaikista nkymist
  *
  *@param user Kyttj
  @param id Viestin ID
  */
  private void deleteMessage(User user, String id) {
    String[] views = this.getViewNames(user);
    for (int i = 0; i < views.length; ++i)
      this.deleteMessageFromView(user, views[i], id);
  }
  
  /**
  *Haetaan nkym
  *
  *@param user Kyttj
  *@param viewname Nkymn nimi
  *@return Nkym tai null jo ei lytynyt
  */
  private View getView(User user, String viewname){
    Document dom = this.db.getDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, viewname);
    if (dom == null) 
      return null;
    NodeList vtags = dom.getElementsByTagName("message");
    String[] namearray = new String[vtags.getLength()];
    Header[] headers = new Header[namearray.length];
    for (int i = 0; i < vtags.getLength(); ++i) {
      namearray[i] = ((Element)vtags.item(i)).getAttribute("id");
      headers[i] = this.getHeader(user, namearray[i]);
    }
    View view = new View(viewname, namearray, headers);
    return view;
  }
  
  //TM EI OLE TEHOKAS TOTEUTUS!!!
  //Mutta header-metodit on muutenkin vain prototyyppi varten!!!
  /**
  *Luodaan viestin otsikkotiedot
  *TM EI TOIMI VIEL
  *
  *@param user Kyttj
  *@param id viestin ID
  */
  private Header getHeader(User user, String id) { //id = messageid
    String sender = "";
    String subject = "";
    String date = "";
    MessageAttributeInterface newAttr = null;
    Document dom = this.getMessage(user, id);
    if (dom == null) //Nin ei pitisi koskaan olla!!!
      return new Header(id, "", "", "");
    Element root = dom.getDocumentElement();
    if (root != null) {
      Element child = (Element) root.getFirstChild();
      while (child != null) {
        if (child.getTagName().equals("item")) {
          if (child.getAttribute("name").equals("Subject")) {
              newAttr = AttributeLoader.newMessageAttributeInstance(child.getAttribute("name"));
              if (newAttr != null) {
                 System.out.println("Setting Subject");
                 newAttr.loadXML(child.getFirstChild());
                 subject = (String) newAttr.getValue();
              }
            }
            if (child.getAttribute("name").equals("From")) {
              newAttr = AttributeLoader.newMessageAttributeInstance(child.getAttribute("name"));
              if (newAttr != null) {
                System.out.println("Setting From");
                 newAttr.loadXML(child.getFirstChild());
                 sender = (String) newAttr.getValue();
              }
            }
            if (child.getAttribute("name").equals("Date")) {
              newAttr = AttributeLoader.newMessageAttributeInstance(child.getAttribute("name"));
              if (newAttr != null) {
                 newAttr.loadXML(child.getFirstChild());
                 Date d = ((Date) newAttr.getValue());
                 if (d != null)
                   date = d.toString();
              }
            }
        }
      child = (Element)child.getNextSibling();
      }
    }
    return new Header(id, sender, subject, date);
  }
  
  /**
  *Haetaan viesti
  *
  *@param user Kyttj
  *@param id Viestin ID
  */
  private Document getMessage(User user, String id){
    return this.db.getDocument(this.db.getUserCollection(user.getName()),
           DatabaseProvider.MESSAGES, id);
  }
  
  /**
  *Haetaan kaikkien nkymien nimet
  *
  *@param user Kyttj
  */
  private String[] getViewNames(User user) {
    try {
    org.apache.xindice.core.Collection col = this.db.getUserCollection(user.getName());
    if (col != null)
      col = col.getCollection(DatabaseProvider.VIEWS);
    if (col != null)
      return col.listDocuments();
    }
    catch(Exception e){System.err.println(e);return null;}
    return null;
  }
  
  
  //Tm on mys hidas
  /**
  *Poistetaan nkym
  *
  *@param user Kyttj
  *@param view Nkymn nimi
  */
  private void deleteView(User user, String view) {
    Document viewdom = this.db.getDocument(this.db.getUserCollection(user.getName()),
          DatabaseProvider.VIEWS, view);
    if (viewdom == null) 
      return;
    NodeList list = viewdom.getElementsByTagName("message");
    for (int i = 0; i < list.getLength(); ++i)
      deleteMessageFromView(user, view, ((Element)list.item(i)).getAttribute("id"));
    
    this.db.deleteDocument(this.db.getUserCollection(user.getName()),
        DatabaseProvider.VIEWS, view);
  }
  
  //Tm pitisi saada toimimaan muulla keinoin
  //getElementById ei toiminut suoraan
  /**
  *Tm on View-dokumenteille tehty getElementById-tyyppinen metodi.
  *Tt ei tarvita, mikli getElementById:n saa toimimaan
  */
  private Element getMessageInView(Document dom, String id) {
    if (dom == null || id == null) return null;
    NodeList list = dom.getElementsByTagName("message");
    Element el;
    for (int i = 0; i < list.getLength(); ++i) {
      el = (Element)list.item(i);
      if (el.hasAttribute("id") && el.getAttribute("id").equals(id)) {
        return el;
      }
    }
    return null;
  }
}


