/*
Tiedosto: AttachmentAttributes.java
Luokat:   AttachmentAttributes

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;
import java.util.Date;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

/**
* Tietovarasto liitetiedoston ominaisuustiedoille.
*/

public class AttachmentAttributes
implements converge.kernel.XMLContentInterface {

    private boolean available = true;
    private long size = -1;
    private Date date = null;
    private String MIMEtype = "text/plain";
    private String description = "unknown";
    private String filename = "unknown.txt";
    private String errorMessage = "";

    /**
    * Oletuskonstruktori
    */
    public AttachmentAttributes() {
        return;
    }

    /**
    * Virhe attribuutteja noudettaessa
    *
    * @param error virheilmoitus
    */
    public AttachmentAttributes(String error) {
        this.available=false;
        this.errorMessage=error;
    }

    /**
    * Kertoo onko liitetiedosto saatavilla (palvelinvirheet ym).
    *
    * @return liitetiedosto ok, true/false
    */
    public boolean getAvailable() {
        return this.available;
    }

    /**
    * Asettaa liitetiedoston saatavilla-tiedon.
    *
    * @param available uusi arvo
    */
    public void setAvailable(boolean available) {
        this.available=available;
    }

    /**
    * Palauttaa tiedon liitetiedoston koosta.
    *
    * @return liitetiedoston koko, -1 jos ei tiedossa
    */
    public long getSize() {
        return this.size;
    }

    /**
    * Asettaa tiedon liitetiedoston koosta.
    *
    * @param size uusi liitetiedoston koko
    */
    public void setSize(long size) {
        this.size=size;
    }

    /**
    * Palauttaa tiedon liitetiedoston viimeisimmst muokkauspivmrst.
    *
    * @return muokkauspivmr
    */
    public Date getDate() {
        return this.date;
    }

    /**
    * Asettaa tiedon liitetiedoston viimeisimmst muokkauspivmrst.
    *
    * @param date uusi muokkauspivmr, tai null jolloin nykyhetki
    */
    public void setDate(Date date) {
        this.date=(date != null ? date : new Date());
    }

    /**
    * Palattaa tiedon liitetiedoston MIME-tyypist.
    *
    * @return MIME-tyyppi
    */
    public String getMIMEType() {
        return this.MIMEtype;
    }

    /**
    * Asettaa tiedon liitetiedoston MIME-tyypist.
    *
    * @param MIMEtype uusi MIME-tyyppi
    */
    public void setMIMEType(String MIMEtype) {
        this.MIMEtype=(MIMEtype != null ? MIMEtype : "");
    }

    /**
    * Palauttaa liitetiedoston kuvauksen.
    *
    * @return liitetiedoston kuvaus
    */
    public String getDescription() {
        return this.description;
    }

    /**
    * Asettaa liitetiedoston kuvauksen.
    *
    * @param description uusi liitetiedoston kuvaus
    */
    public void setDescription(String description) {
        this.description=(description != null ? description : "");
    }

    /**
    * Palauttaa liitetiedoston tiedostonimen.
    *
    * @return tiedostonimi
    */
    public String getFilename() {
        return this.filename;
    }

    /**
    * Asettaa liitetiedoston tiedostonimen.
    *
    * @param filename uusi liitetiedoston tiedostonimi
    */
    public void setFilename(String filename) {
        this.filename=(filename != null ? filename : "unknown.txt");
    }

    /**
    * Palauttaa liitetiedostoa haettaessa tulleen virheilmoituksen.
    *
    * @return virheilmoitus
    */
    public String getErrorMessage() {
        return this.errorMessage;
    }

    /**
    * Asettaa liitetiedostoa haettaessa tulleen virheilmoituksen.
    *
    * @param filename uusi virheilmoitus
    */
    public void setErrorMessage(String error) {
        this.errorMessage=(error != null ? error : "");
    }

    /**
    * Kertoo ovatko parametrina annetun AttachmentAttributes-olion tiedot
    * saman kuin tmn olion.
    *
    * @param verrattava AttachmentAttributes-olio
    */
    public boolean isSame(AttachmentAttributes cmp) {
        return this.available == cmp.available
            && this.size == cmp.size
            && this.date.equals(cmp.date)
            && this.MIMEtype.equals(cmp.MIMEtype)
            && this.description.equals(cmp.description)
            && this.filename.equals(cmp.filename)
            && this.errorMessage.equals(cmp.errorMessage);
    }

    /**
    * Muodostaa AttachmentAttributes-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        Document doc = parent.getOwnerDocument();

        Element attr = doc.createElement("attributes");
        parent.appendChild(attr);

        Element e1 = doc.createElement("available");
        e1.setAttribute("value", this.available ? "1" : "0");
        attr.appendChild(e1);

        Element e2 = doc.createElement("size");
        Long l = new Long(this.size);
        e2.setAttribute("value", l.toString());
        attr.appendChild(e2);

        Element e3 = doc.createElement("date");
        if(this.date == null)
            e3.setAttribute("value","0");
        else
            e3.setAttribute("value", new Long(this.date.getTime()).toString());
        attr.appendChild(e3);

        Element e4 = doc.createElement("mimetype");
        e4.setAttribute("value", this.MIMEtype);
        attr.appendChild(e4);

        Element e5 = doc.createElement("description");
        e5.setAttribute("value", this.description);
        attr.appendChild(e5);

        Element e6 = doc.createElement("filename");
        e6.setAttribute("value", this.filename);
        attr.appendChild(e6);

        Element e7 = doc.createElement("errormessage");
        e7.setAttribute("value", this.errorMessage);
        attr.appendChild(e7);
    }

    /**
    * Muodostaa annetusta XML-elementist AttachmentAttributes-olion.
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        Element content = (Element)myContent;
        if(!content.getTagName().equals("attributes"))
            return false;

        Element child = (Element)content.getFirstChild();
        while (child != null) {
            if(child.getTagName().equals("available"))
                this.available=child.getAttribute("value").equals("1");
            else if(child.getTagName().equals("size"))
                this.size=new Long(child.getAttribute("value")).longValue();
            else if(child.getTagName().equals("date"))
                if(child.getAttribute("value").equals("0"))
                    this.date=null;
                else
                    this.date=new Date(new Long(child.getAttribute("value")).longValue());
            else if(child.getTagName().equals("mimetype"))
                this.MIMEtype=child.getAttribute("value");
            else if(child.getTagName().equals("description"))
                this.description=child.getAttribute("value");
            else if(child.getTagName().equals("filename"))
                this.filename=child.getAttribute("value");
            else if(child.getTagName().equals("errormessage"))
                this.errorMessage=child.getAttribute("value");

            child = (Element) child.getNextSibling();
        }
        return true;
    }
}
