/*
Tiedosto: DataSource.java
Luokat:   DataSource

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;

/**
* DataSource on abstrakti ylluokka kaikille tiedonlhteille. Luokka tarjoaa aliluokilleen
* joitakin tiettyj yleisi metodeja, mutta suurin osa metodeista pit tehd tiedonlhdekohtaisesti.
* DataSource-luokasta ei tehd koskaan ilmentymi, ainoastaan sen aliluokista.
*/
public abstract class DataSource
implements converge.kernel.XMLContentInterface {

    protected String dataSourceName=null;
    protected String latestErrorMessage=null;
    protected int consecutiveFailures=0; //perkkisi virheellisi yhteydenottopyyntj

    /**
    * Palauttaa DataSource-ilmentymn luonnollisen nimen, esim. "Yliopiston IMAP"
    *
    * @return DataSourcen nimi
    */
    public String getName() {
        return this.dataSourceName;
    }

    /**
    * Asettaa DataSource-ilmentymn luonnollisen nimen, esim. "Yliopiston IMAP"
    *
    * @param newName uusi DataSourcen nimi
    */
    public void setName(String newName) {
        this.dataSourceName=newName;
    }

    /**
    * Kertoo onko DataSource kytettviss (palvelinvirheet).
    *
    * @return true jos ok, false jos ongelmia
    */
    public boolean isValid() {
        return (latestErrorMessage==null);
    }

    /**
    * Kertoo tt DataSourcea viimeksi kytettess tulleen virheilmoituksen.
    *
    * @return virheilmoitus, null jos ei virhett
    */
    public String getErrorMessage() {
        return this.latestErrorMessage;
    }
    
    /**
    * Asettaa tt DataSourcea viimeksi kytettess tulleen virheilmoituksen.
    *
    * @param err virheilmoitus, null jos ei virhett
    */
    public void setErrorMessage(String err) {
        this.latestErrorMessage=err;
    }

    /**
    * Palauttaa uuden halutuntyyppisen DataSource-olion.
    *
    * @return uusi DataSource-olio
    */
    public static DataSource getDataSource(String name) {
        // toistaiseksi vain yksi DataSource, joten tm riitt
        return new converge.service.IMAP();
    }

    /**
    * Hakee uudet saapuneet viestit.
    *
    * @param user kyttjolio
    */
    public abstract void getNewData(converge.kernel.User user);

    /**
    * Palauttaa DataSource-olion tyypin
    *
    * @return olion tyyppi
    */
    public abstract String getType();

    /**
    * Palauttaa tiedon siit, pystyyk tm DataSource ksittelemn annettua pointteria.
    *
    * @param URL pointteri
    * @return pystyyk tm DataSource noutamaan tmn URLin (teoriassa)
    */
    public abstract boolean canHandle(String URL);

    /**
    * Noutaa liitetiedoston osoitteesta URL.
    *
    * @param URL pointteri
    * @param testOnly tarkistetaanko vain liitetiedoston olemassaolo
    * @return tiedostokahvan ja mahdollisen virheilmoituksen sisltv PipeMessage
    */
    public abstract PipeMessage getAttachment(String URL, boolean testOnly);

    /**
    * Muodostaa DataSource-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public abstract void createXML(org.w3c.dom.Node parent);

    /**
    * Muodostaa annetusta XML-elementist DataSource-olion.
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public abstract boolean loadXML(org.w3c.dom.Node myContent);
}
