/*
Tiedosto: HTTP.java
Luokat:   HTTP

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti
  14.12.2002  Poistettu getAttachment ja listty getAttachmentAttributes

*/
package converge.service;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import converge.kernel.Log;
import javax.net.ssl.HttpsURLConnection;

/**
* HTTP hakee WWW-liitetiedostojen tiedot WWW-sivuilta.
*/

public class HTTP  {

    /**
    * Palauttaa liitetiedoston ominaisuudet (koko, pivmr ym).
    *
    * @param URL liitetiedoston osoite
    * @return AttachmentAttributes-olio
    */
    public static AttachmentAttributes getAttachmentAttributes(String URL) {

        if(! (URL.substring(0,7).equalsIgnoreCase("http://") || URL.substring(0,8).equalsIgnoreCase("https://")))
            return new AttachmentAttributes("Invalid protocol specified");

        HttpURLConnection input = null;

        try {
            URL url = new URL(URL);
            AttachmentAttributes aa = new AttachmentAttributes();
            if(URL.substring(0,8).equalsIgnoreCase("https://")) {
                HttpsURLConnection.setDefaultSSLSocketFactory(new ConvergeSSLSocketFactory());
                input = (HttpsURLConnection)url.openConnection();
            } else
                input = (HttpURLConnection)url.openConnection();
            input.setRequestMethod("HEAD");
            input.connect();
            //System.out.println("HTTP error: " + input.getResponseCode() + ": " + input.getResponseMessage());
            if(input.getResponseCode() < 200 || input.getResponseCode() > 299) {
                input.disconnect();
                return new AttachmentAttributes("HTTP error: " + input.getResponseCode() + ": " + input.getResponseMessage());
            }
            aa.setDate(new Date(input.getLastModified()));
            aa.setSize(input.getContentLength());
            aa.setMIMEType(input.getContentType());
            input.disconnect();
            return aa;
        } catch(Exception e) {
            input.disconnect();
            System.out.println("HTTP exception: " + e);
            Log.stack(e,"HTTP");
            return new AttachmentAttributes("Error during http/https session: "+e);
        }
    }

    // testimetodi
    public static void main(String[] args) {
        String[] test ={"http://www.miuku.net", "http://www.helsinki.fi/", "https://www.helsinki.fi/" };
        for(int i=0;i<test.length;i++) {
            System.out.println("\nTesting " + test[i]);
            AttachmentAttributes aa = getAttachmentAttributes(test[i]);
            System.out.println("Available " +aa.getAvailable());
            System.out.println("Size " + aa.getSize());
            System.out.println("Date " + aa.getDate().toString());
            System.out.println("MIME " + aa.getMIMEType());
            /* tulosti:
            Testing http://www.miuku.net
            HTTP error: 200: OK
            Available true
            Size 3244
            Date Sun Sep 01 00:53:26 EEST 2002
            MIME text/html; charset=iso-8859-1

            Testing http://www.helsinki.fi/
            HTTP error: 200: OK
            Available true
            Size -1
            Date Thu Dec 19 16:25:08 EET 2002
            MIME text/html

            Testing https://www.helsinki.fi/
            HTTP error: 200: OK
            Available true
            Size 574
            Date Thu Jun 08 13:11:43 EEST 2000
            MIME text/html
            */
        }
    }
}
