/*
Tiedosto: IMAP.java
Luokat:   IMAP

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/
package converge.service;
import java.util.Date;
import java.util.Properties;
import javax.mail.Multipart;
import javax.mail.Folder;
import javax.mail.Store;
import javax.mail.Session;
import javax.mail.search.MessageIDTerm;
import java.security.Security;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import converge.kernel.Log;
import webcore.HtmlUtils;

/**
* IMAP on DataSource-luokan aliluokka ja laajentaa DataSourcea toteuttamalla tuen
* IMAP-postilaatikoille.
*/

public class IMAP
extends DataSource {

    protected String serverAddress;
    protected String serverLogin;
    protected String serverPassword;
    protected Date lastChecked;
    protected boolean useSSL;

    /**
    * Oletuskonstruktori. Kytetn vain loadXML:n yhteydess.
    */
    public IMAP() {
        this.dataSourceName="Unknown"; /* peritty DataSourcesta */
        this.serverAddress="unknown";
        this.serverLogin="unknown";
        this.serverPassword="unknown";
        this.useSSL=true;
    }

    /**
    * Luo uuden IMAP-olion.
    *
    * @param dataSourceName luonnollinen nimi, esim. "Yliopiston IMAP"
    * @param serverAddress  palvelimen nimi, esim. "ml.mappi.helsinki.fi"
    * @param serverLogin    kyttjtunnus
    * @param serverPassword salasana
    */
    public IMAP(String dataSourceName, String serverAddress, String serverLogin, String serverPassword) {
        this.dataSourceName=dataSourceName; /* peritty DataSourcesta */
        this.serverAddress=serverAddress;
        this.serverLogin=serverLogin;
        this.serverPassword=serverPassword;
        this.useSSL=true;
    }

    /**
    * Hakee uudet saapuneet viestit. Kynnist IMAPMessageThreadin viestien hakemiseksi.
    *
    * @param user kyttjolio
    */
    public void getNewData(converge.kernel.User user) {
        // haetaan postit vain kun consecutiveFailures on:
        // 0, 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 768, 1024, 1280, 1536, 1792, 2048 ..
        if(this.consecutiveFailures == 0 ||
           this.consecutiveFailures == 1 ||
           this.consecutiveFailures == 2 ||
           this.consecutiveFailures == 4 ||
           this.consecutiveFailures == 8 ||
           this.consecutiveFailures == 16 ||
           this.consecutiveFailures == 32 ||
           this.consecutiveFailures == 64 ||       // n. 1h
           this.consecutiveFailures == 128 ||      // n. 2h
           this.consecutiveFailures % 256 == 0) {  // n. joka 4h
            IMAPMessageThread imt = new IMAPMessageThread(this, user);
            /* kynnistetn sie */
            imt.start();
        } else {
            System.out.println("IMAP: skipping " + this.serverAddress + " because error count = " +
                this.consecutiveFailures);
            this.consecutiveFailures++;
        }
    }

    /**
    * Palauttaa palvelimen osoitteen.
    *
    * @return palvelimen osoite
    */
    public String getServerAddress() {
        return this.serverAddress;
    }

    /**
    * Palauttaa kyttjnimen
    *
    * @return kyttjnimi
    */
    public String getServerLogin() {
        return this.serverLogin;
    }

    /**
    * Palauttaa DataSourcen tyypin.
    *
    * @return DataSourcen tyyppi, vakio "IMAP"
    */
    public String getType() {
        return "IMAP";
    }

    /**
    * Kertoo onko parametrina annettu pointteri sellainen, jonka
    * tm DataSource-olio voisi hakea.
    *
    * @param URL pointteri liitetiedostoon
    * @return pystyyk tm DataSource hakemaan mainitun liitetiedoston
    */
    public boolean canHandle(String URL) {
        if(! (URL.substring(0,7).equals("imap://")))
            return false;
        IMAPURLProperties iup = new IMAPURLProperties(URL);
        if(iup.protocol == null)
            //virheellinen URL
            return false;

        if(this.serverAddress.equals(iup.server)
        && this.serverLogin.equals(iup.username))
            return true;
        return false;
    }

    /**
    * Muodostaa IMAP-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        Document doc = (Document)parent;
        Element ds = doc.createElement("IMAP");
        parent.appendChild(ds);
        ds.setAttribute("name", this.getName());

        Element addr = doc.createElement("address");
        addr.appendChild(doc.createTextNode(this.serverAddress));
        ds.appendChild(addr);

        Element login = doc.createElement("login");
        login.appendChild(doc.createTextNode(this.serverLogin));
        ds.appendChild(login);

        Element password = doc.createElement("password");
        password.appendChild(doc.createTextNode(
            converge.kernel.Crypt.encode("convergepasswd",this.serverPassword.getBytes())));
        ds.appendChild(password);

        Element lastchecked = doc.createElement("lastchecked");
        if(this.lastChecked == null)
            lastchecked.appendChild(doc.createTextNode("0"));
        else
            lastchecked.appendChild(doc.createTextNode(new Long(this.lastChecked.getTime()).toString()));
        ds.appendChild(lastchecked);

        Element usessl = doc.createElement("usessl");
        usessl.appendChild(doc.createTextNode(this.useSSL ? "1" : "0"));
        ds.appendChild(usessl);
    }

    /**
    * Muodostaa annetusta XML-elementist IMAP-olion.
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {

        if ( !((Element)myContent).getNodeName().equals("IMAP") ) {
            return false;
        }
        this.dataSourceName=((Element)myContent).getAttribute("name");

        myContent=myContent.getFirstChild();
        if ( myContent == null ) {
            return false;
        }

        while(myContent != null) {
            if(myContent.getNodeName().equals("address"))
                this.serverAddress=myContent.getFirstChild().getNodeValue();
            else if(myContent.getNodeName().equals("login"))
                this.serverLogin=myContent.getFirstChild().getNodeValue();
            else if(myContent.getNodeName().equals("password"))
                this.serverPassword=converge.kernel.Crypt.decode("convergepasswd",
                                    myContent.getFirstChild().getNodeValue());
            else if(myContent.getNodeName().equals("lastchecked"))
                if(myContent.getFirstChild().getNodeValue().equals("0"))
                    this.lastChecked=null;
                else
                    this.lastChecked=new Date(new Long(myContent.getFirstChild().getNodeValue()).longValue());
            else if(myContent.getNodeName().equals("usessl"))
                this.useSSL=myContent.getFirstChild().getNodeValue().equals("1");
            myContent=myContent.getNextSibling();
        }
        return this.serverAddress != null && this.serverLogin != null
            && this.serverPassword != null;

    }

    /**
    * Noutaa liitetiedoston IMAP-postilaatikosta.
    *
    * @param URL  liitetiedoston osoite, esim. imap://kayttaja@imap.palvelin.fi/<pine.osf.4.30.1322423@sirppi.helsinki.fi>/1-liitetiedostonnimi.txt
    * @param testOnly tarkistetaanko vain liitetiedoston olemassaolo
    * @return tiedostokahvan ja mahdollisen virheilmoituksen sisltv PipeMessage
    */
    public PipeMessage getAttachment(String URL, boolean testOnly) {
        IMAPURLProperties iup = new IMAPURLProperties(URL);
        if(!iup.protocol.equals("imap"))
            return new PipeMessage("Pointer does not point to an IMAP mailbox");

        try {
            Session session;
            if(this.useSSL) {
                final String SSL_FACTORY = "converge.service.ConvergeSSLSocketFactory";
                Security.setProperty("ssl.SocketFactory.provider","ConvergeSSLSocketFactory");
                Properties props = System.getProperties();
                props.setProperty("mail.imap.socketFactory.class",SSL_FACTORY);
                /* jos palvelin ei ymmrr SSL: niin ei edes yritet kirjoittaa SSL-juttuja SSL-porttiin */
                props.setProperty("mail.imap.socketFactory.fallback","false");
                props.setProperty("mail.imap.port","993");
                props.setProperty("mail.imap.socketFactory.port","993");
                session = Session.getInstance(props);
            } else
                session = Session.getInstance(new Properties(), null);
            Store store = session.getStore("imap");
            store.connect(this.serverAddress,this.serverLogin,this.serverPassword);
            Folder folder = store.getFolder("INBOX"); //tm voi kai joskus olla jotain muutakin!
            folder.open(Folder.READ_ONLY);
            javax.mail.Message message[] = folder.search(new MessageIDTerm(iup.msgid));
            if(message.length == 0) {
                folder.close(false);
                store.close();
                return new PipeMessage("Message " + webcore.HtmlUtils.convertTag2Entity(iup.msgid) + " was not found");
            }
            for(int i=0;i<message.length;i++) {
                Object content = message[i].getContent();
                if (! (content instanceof Multipart))
                    continue;
                PipeMessage pm = Email.getAttachment(((Multipart)content).getBodyPart(iup.msgnum),iup.file,
                                folder, store, testOnly);
                if(pm != null)
                    return pm;
            }
            folder.close(false);
            store.close();
            return new PipeMessage("Message was found but it didn't have an attachment");
        } catch(Exception e) {
            System.out.println("IMAP exception: " + e);
            Log.stack(e,"IMAP");
            return new PipeMessage("IMAP error: " + e);
        }
    }


//DEBUG **********************************
    public static void main(String[] args) {
        IMAP i = new IMAP("Converge-1","imap.converge.miuku.net","testi1","converge");
        converge.attribute.AttributeLoader.loadClasses();
        i.getNewData(null);

        System.out.println("IMAP: alkuperinen datasource");
        org.w3c.dom.Document doc = converge.kernel.XMLUtil.newXMLDocument();
        i.createXML(doc);
        converge.kernel.XMLUtil.outputPrinting(doc,System.out);

        System.out.println("IMAP: datasourcen kopio");
        IMAP i2 = new IMAP();
        i2.loadXML(doc.getFirstChild());
        org.w3c.dom.Document doc2 = converge.kernel.XMLUtil.newXMLDocument();
        i2.createXML(doc2);
        converge.kernel.XMLUtil.outputPrinting(doc2,System.out);


    }
//DEBUG **********************************
}
