/*
Tiedosto: IMAPURLProperties.java
Luokat:   IMAPURLProperties

Tekij:   Anssi Johansson
Luonti:   12.12.2002

Muutokset:
  12.12.2002  Luokan luonti Timon koodin pohjalta
*/

package converge.service;

/**
* Luokka joka pilkkoo annetun IMAP-pointterimerkkijonon osiksi ja tarkistaa syntaksin.
*/
public class IMAPURLProperties {

    protected String protocol=null, username=null, server=null, msgid=null, file=null;
    protected int msgnum=0;

    /**
    * Tarkistaa IMAP-pointterimerkkijonon.
    *
    * @param pointer liitetiedoston osoite, esim "imap://tunnus@palvelin/<messageviesti@432874298742>/1-liitetiedosto.txt"
    */
    protected IMAPURLProperties(String pointer) {
        if (pointer.indexOf("://") < 0 || pointer.indexOf(":///") >= 0)
            return;
        if (pointer.indexOf("@") < 0 || pointer.indexOf("@") > pointer.indexOf("/<"))
            return;
        if (pointer.indexOf("/<") < 0)
            return;
        int msgidEnd = pointer.lastIndexOf(">/") + 1;
        if (msgidEnd < 0)
            return;

        int msgnumStart = msgidEnd+2;
        int fileStart = msgnumStart;

        try {
            char c = pointer.charAt(fileStart);
            while(c == '0' || c == '1' || c == '2' ||
                  c == '3' || c == '4' || c == '5' ||
                  c == '6' || c == '7' || c == '8' ||
                  c == '9')
                  fileStart++;
        } catch(Exception e) { //ylivuoto
            return;
        }
        if(pointer.charAt(fileStart) != '-')
            return;
        fileStart++;

        try {
            msgnum = Integer.parseInt(pointer.substring(msgnumStart-1,fileStart-1));
        } catch (NumberFormatException e) {
            return;
        }

        protocol = pointer.substring(0, pointer.indexOf("://"));
        if(!(protocol.equals("imap"))) {
            protocol=null;
            msgnum=0;
            return;
        }
        username = pointer.substring(pointer.indexOf("://") + 3, pointer.indexOf("@"));
        server = pointer.substring(pointer.indexOf("@") + 1, pointer.indexOf("/<"));
        msgid = pointer.substring(pointer.indexOf("/<") + 1, msgidEnd);
        file = pointer.substring (fileStart, pointer.length());
    }

    public static void main(String[] args) {
        final String test = "imap://testi1@imap.converge.miuku.net/1-mm.txt";
        IMAPURLProperties iup = new IMAPURLProperties(test);
        System.out.println(test);
        System.out.println(iup.protocol);
        System.out.println(iup.username);
        System.out.println(iup.server);
        System.out.println(iup.msgid);
        System.out.println(iup.file);
        System.out.println(iup.msgnum);
    }
}
