/*
Tiedosto: Contact.java
Luokat:   Contact

Tekij:   Olli Pettay
Luonti:   13.12.2002


Muutokset:
 
13.12.2002 Olli
            Luokan luonti
15.12.2002 Joni
            Pieni muutoksia & kommentointia        
                      
*/

package webcore;
import converge.kernel.*;
import converge.clientmanager.*;
import converge.service.*;
import java.io.*;
import java.util.*;
import webcore.webserver.*;

/**
Kontaktilistan ksittely.
<br><br>
Tm luokka toteuttaa WWW-kyttliittymn sen osan, jossa kyttj voi list kontaktilistaansa kontakteja, sek poistaa niit.
<br><br>
Mahdolliset HTTP-parametrit:
name - listtvn kontaktin nimi
email - listtvn kontaktin shkpostiosoite 
delete-contact - poistettavan kontaktin nimi
*/
public class Contact implements WebRequestable{
    static ClientManager cm = ClientManager.getInstance();
    User user;
    long receivedSessionId;
    boolean hasCookie;
    boolean emergencyExit;
    
    /**
    Ksitelln HTTP-pyynt.

    @param httpRequest ksiteltv HTTP-pyynt
    */
    public void processRequest(HTTPRequest httpRequest){
        PrintWriter pw = null;
        boolean messageSent = false;

        // aluksi:  haetaan cookie, sen perusteella user

        String tmpCookie = httpRequest.getCookie("session-id");
        if (tmpCookie != null){
            hasCookie = true;
            receivedSessionId = Long.parseLong(tmpCookie);
        }
        user = cm.getUserBySessionId(receivedSessionId);

        // tarkistus session vanhentumisen/muun virheen varalta
        if (user == null)
            emergencyExit = true;

        // uuden kontaktin lisys:

        else if (httpRequest.getFormVar("name") != null &&
            httpRequest.getFormVar("email") != null && user.writeLock(this)) {
            ContactList cl = user.getContactList();
            converge.kernel.Contact c = new converge.kernel.Contact(
            HtmlUtils.removeQuot(HtmlUtils.removeTag(httpRequest.getFormVar("name"))));
            c.set("email", HtmlUtils.normalize(httpRequest.getFormVar("email")));
            cl.addContact(c);
            user.releaseLock(this);
        }

        // olemassaolevan kontaktin poisto:

        else if (httpRequest.getFormVar("delete-contact") != null &&
                user.writeLock(this)) {
             user.getContactList().removeContact(httpRequest.getFormVar("delete-contact"));
             user.releaseLock(this);
        }


        try {
            pw = httpRequest.getPrintWriter();

            if (emergencyExit)
                HtmlUtils.printEmergencyExitPage(pw);

            else{
                pw.println(htmlHeader);
                pw.print(HtmlUtils.startPage("Contact list"));
                pw.println("<BODY>");

                // tulostetaan 'navigaatiopalkki'
                HtmlUtils.printNavigationBar(pw);

                // Sivun otsikko ja kyttjnimi jolla on kirjauduttu jrjestelmn
                HtmlUtils.printPageInfo(pw, user.getName(), "Contacts");

                // tulostetaan kontaktilistan sislt,
                //sek lomake jolla kontakteja voi poistaa

                if (user.readLock(this)) {
                    ContactList cl = user.getContactList();
                    Set names = cl.getContactNames();
                    if (!names.isEmpty()) {
                        pw.println("<table cellspacing='5' border='0' style='border: solid 1px;'>");
                        pw.println("<tr><td><b>Name</b></td><td width='15'></td>"
                            +"<td><b>Email</b></td><td width='15'></td><td></td></tr>");
                        for (Iterator it = names.iterator(); it.hasNext();) {
                            converge.kernel.Contact c = cl.getContact((String)it.next());
                            pw.println("<tr><td>" +c.getName() + " </td><td></td>"
                                    +"<td> " + c.get("email") + "</td>");
                            pw.println("<td></td><td>");
                            pw.println("<form method='post' action='webcore.Contact'>");
                            pw.println("<input type='hidden' name='delete-contact' value='"
                                    + c.getName()+"'>");
                            pw.println("<input type='submit' name='delete' value='delete'></form>");
                            pw.println("</td>");
                            pw.println("</tr>");
                        }
                        pw.println("</table>");
                    }

                    user.releaseLock(this);
                }

                // lomake uusien kontaktien syttmist varten
                printAddContactForm(pw);

                //testausta varten:
                HtmlUtils.printTestingStuff(pw, httpRequest);

                // sivun loppu:
                HtmlUtils.printEndOfPage(pw);
            }
        }

        catch (Exception e) {
            e.printStackTrace(pw);
        }

        finally {
            try{httpRequest.close();}
        catch (java.io.IOException e2) {e2.printStackTrace(pw);}
        }

        if (user != null)
            user.releaseLock(this);
    }


    // apumetodeita

    /**
    Tulostaa annettuun PrintWriteriin HTML-lomakkeen jolla voi list uusia kontakteja.
    */
    private static void printAddContactForm(PrintWriter pw){
        pw.println("<div style='margin-top: 1em;'>");
        pw.println("<h4>Add a new contact:</h4>");
        pw.println("<form method='post' action='webcore.Contact'><table>");
        pw.println("<tr><td>Name:</td><td><input size='30' type='text' "
                    +"name='name'></td></tr>");
        pw.println("<tr><td>Email:</td><td><input size='30' type='text' "
                    +"name='email'></td></tr>");
        pw.println("<tr><td></td><td><input type=submit value='Add contact'>");
        pw.println("<input type='hidden' name='addform' value='sent'></td></tr>");
        pw.println("</table></form></div>");
    }

}
