/*
Tiedosto: HtmlUtils.java
Luokat:   HtmlUtils

Tekij:   Joni Karppinen
Luonti:   6.12.2002

Muutokset:

6.12.2002     Joni    
           Luokka luotu, pari HTML-apumetodia alustavasti tehty;
           HTML-sivun alku, HTTP-headerien luomiseen erilaisia metodeja 
13.12.2002    Olli
           Erilaisia apumetodeja
15.12.2002    Joni    
           Parannuksia mm. printNavigationBar -metodiin
18.12.2002    Joni
            Javadoccien kirjoitusta, muuta siistimist ja yhtenistmist    
19.12.2002    Joni
            printTestingStuff -metodi tulostaa vain jos asetustiedossa niin sanotaan            

*/

package webcore;
import java.io.*;
import webcore.webserver.*;

/**
Kirjastoluokka, johon on kertty hydyllisi apumetodeja muiden webcore-pakkauksen
luokkien kyttn.
<br><br>
Tlt lytyy mm. metodit navigaatiopalkin tulostamiseen sek  
HTTP-otsikkotietojen asettamiseen. 
*/
public final class HtmlUtils{

    /**
    Asetukset
    */
    private static java.util.Properties settings = new java.util.Properties();
    
    /**
    Tulostaako printTestingStuff kutsuttaessa mitn
    */
    private static boolean printIt = false;
    
    /** 
    Staattinen alustuslohko, jossa luetaan asetustiedostosta asetuksia.
    (Tss tapauksessa vain yksi asetus.)    
    */    
    static{                
        try{            
            settings.load(new FileInputStream(new File("config/clientmanager.cfg")));
            printIt = Boolean.valueOf(settings.getProperty("testing-stuff","false")
                                      ).booleanValue();
        }
        catch (Exception e){
            System.out.println("HTMLTUTILS-alustuslohko: " + e);
            printIt = false;                        
        }        
    }
    
    
    /**
    Estetn ilmentyminen luonti.
    */
    private HtmlUtils(){        
    }
    
    /**
    Palauttaa HTML-sivun alkuosan Stringin.
    @param title Sivun otsikko
    */
    public static String startPage(String title){

        StringBuffer s = new StringBuffer();
        s.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
        s.append("<html>\n<head>\n");
        s.append("<link rel='stylesheet' title='Converge basic style' "
                +"type='text/css' href='../converge.css'>\n");
        s.append("<title>"+title+" // Convergence of messaging</title>\n");
        s.append("</head>\n");
        return (s.toString());
    }

    
    /**
    Palauttaa HTML-sivun alkuosan Stringin. Tmn avulla voidaan tehd uudelleenohjaus halutulle sivulle.

    @param title Sivun otsikko
    @param redirect Tiedoston nimi, johon tehdn uudelleenohjaus, esim. "Main"
    */
    public static String startPage(String title, String redirect){

        StringBuffer s = new StringBuffer();
        s.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
        s.append("<html>\n<head>\n");
        s.append("<link rel='stylesheet' title='Converge basic style' "
                +"type='text/css' href='../converge.css'>\n");
        s.append("<meta http-equiv='refresh' content='1;url=webcore."+redirect+"'>\n");
        s.append("<title>"+title+" // Convergence of messaging</title>\n");
        s.append("</head>\n");
        return (s.toString());
    }
        
    /**
    Tulostaa annettuun PrintWriteriin HTML-sivun lopun
    @param pw PrintWriter, johon tulostetaan
    */
    public static void printEndOfPage(PrintWriter pw){
            pw.print("\n</body></html>\n");
    }
    
    /**
    Tulostaa annettuun PrintWriteriin sivun otsikon ja tiedon mill tunnuksella 
    ollaan kirjauduttu jrjestelmn     
    
    @param pw PrintWriter, johon tulostetaan
    */
    public static void printPageInfo(PrintWriter pw, String userName, String pageTitle){
        pw.print("<p>Logged in as: <b>"+userName+"</b></p>\n");
        pw.print("<h3>"+pageTitle+"</h3>\n");    
    }
    
    
    //  HTTP-headerien tulostukset

    //  Cookie-syntaksi...
    //  http://wp.netscape.com/newsref/std/cookie_spec.html  <-- ohjetta
    //
    //  Set-Cookie: NAME=VALUE; expires=DATE; path=PATH; domain=DOMAIN_NAME; secure
    //  Set-cookie: muuttuja=arvo; path=/cgi/bin; expires=GMT date; domain=DOMAIN;
    //
    //  Wdy, DD-Mon-YYYY HH:MM:SS GMT


    /**
    Palauttaa HTTP-headerin joka lhett selaimelle cookiena parametrina saatavan session-ID:n

    @param sessionId cookien sisllksi tuleva session-id
    */
    public static String cookieHeader(long sessionId){
        StringBuffer s = new StringBuffer();
        s.append("HTTP/1.0 200 OK\r\n");
        s.append("Content-Type: text/html\r\n");
        s.append("Set-cookie: session-id="+sessionId+";");
        s.append("\r\n\r\n");
        return s.toString();
    }

    /**
    Palauttaa HTTP-headerin joka poistaa selaimella (mahdollisesti) olevan session-id -cookien. (Tm tapahtuu siten, ett cookien vanhentumisaika asetetaan menneisyyteen.)
    <br><br>
    [Set-Cookie: name=deleted; expires=Thu, 01-Jan-1970 00:00:00 GMT;]
    */
    public static String removeCookieHeader(){
        StringBuffer s = new StringBuffer();
        s.append("HTTP/1.0 200 OK\r\n");
        s.append("Content-Type: text/html\r\n");
        //s.append("Set-cookie: session-id=0;");
        s.append("Set-cookie: session-id=deleted; expires="
                    +"Thu, 01-Jan-1970 00:00:00 GMT;");
        s.append("\r\n\r\n");
        return s.toString();
    }

    /**
    Palauttaa HTTP-protokollan mukaiset WWW-palvelimen otsikkotiedot liitetiedostojen lhetyst varten.

    @param contentType lhetettvn tiedoston tyyppi, esim. "image/jpeg"
    @param length      lhetettvn tiedoston koko
    @param fileName    tiedostonimi
    @return HTTP-otsikkotiedot
    */
    public static String printHeader(String contentType, long length, String fileName) {
        return "HTTP/1.0 200 OK\r\nContent-Type: "+contentType+"\r\nContent-Length: "
               +length+"\r\nContent-Disposition: attachment; filename="+fileName+"\r\n\r\n";
    }

    /**
    Palauttaa HTTP-protokollan mukaiset WWW-palvelimen otsikkotiedot liitetiedostojen lhetyst varten.

    @param contentType lhetettvn tiedoston tyyppi, esim. "image/jpeg"
    @param fileName    tiedostonimi
    @return HTTP-otsikkotiedot
    */
    public static String printHeader(String contentType, String fileName) {
        return "HTTP/1.0 200 OK\r\nContent-Type: "+contentType+"\r\n"
               +"Content-Disposition: attachment; filename="+fileName+"\r\n\r\n";
    }



    /**
    Tulostaa annettuu PrintWriteriin kyttliittymn "navigaatiopalkin", eli lhes joka sivulle tulevan linkkikokoelman, jonka avulla voi siirty jrjestelmn eri
    toimintoihin.
    <br><br>
    Parannusidea: 
    Parametrina voitaisiin saada se sivu jolla nyt ollaan. olisi ehk kytettvyyden kannalta hyv ratkaisu tulostaa se esim. eri vrill...
    [ class='current' ja css-tiedostoon vastaava snt. ]
    @param pw PrintWriter, johon tulostetaan
    */
    public static void printNavigationBar(PrintWriter pw){

        StringBuffer s = new StringBuffer();

        s.append("<div class='navigation'>\n"
                +"<a class='button' title='Main page with list of views'"
                    +"href='webcore.Main'>Main</a> | \n"
                +"<a class='button' title='Add/remove datasources (IMAP-servers)'"             
                    +"href='webcore.Settings'>DataSources</a> | \n"
                +"<a class='button' title='Manage your context models'"
                    +"href='webcore.ContextModel'>ContextModels</a> | \n"
                +"<a class='button' title='Manage your profiles'"
                    +"href='webcore.Profile'>Profiles</a> | \n"
                +"<a class='button' title='Manage your contact list'"
                    +"href='webcore.Contact'>Contactlist</a> | \n"
                +"<a class='button' title='Send new messages' "
                    +"href='webcore.Write'>Write</a> | \n"
                +"<a class='button' title='Log out of the system' "
                    +"href='webcore.Login?logout=true'>Log out</a> \n"
                +"</div> \n");

        pw.print(s.toString());
    }


    /**
    Palauttaa selaimelle sivun joka uudelleenohjautuu Login-sivulle ja poistaa
    samalla  session-id -keksin. Tt voi kutsua jos viite user-olioon on null
    sellaisilla    sivuilla joilla aina tarvitaan validi User-olio.
    
    @param pw PrintWriter, johon tulostetaan
    */
    public static void printEmergencyExitPage(PrintWriter pw){
        pw.print(removeCookieHeader());     // poistetaan session-id -keksi
        pw.print("<html><head>\n");
        pw.print("<meta http-equiv='refresh' content='2;url=webcore.Login'>\n");
        pw.print("<link rel='stylesheet' title='Converge basic style' "
                +"type='text/css' href='../converge.css'>\n");
        pw.print("</head><body>"
               //+" <h4>Sessio vanhentunut / joku muu vialla</h4> (eli user oli null...) \n");
               //+" <h4>Session has expired / user account has been deleted "
               //+"/ something else is wrong</h4> (i.e. user was null...) \n");
             +"<h3>Logout in progress...</h3>"
             +"Possible causes: <ul><li>Session has expired</li><li>User account has been deleted</li><li>Something else is wrong</li></ul>");  
        pw.print("</body></html>");
    }

    /**
    Testauksessa kytettv metodi, joka tulostaa annettuun PrintWriteriin annetun
    HTTPRequestin mukana tulevat HTTP-parametrit ja keksit.

    @param pw PrintWriter, johon tulostetaan
    @param httpRequest HTTP-pyynt josta saadaan tulostettavat parametrit ja keksit
    */
    public static void printTestingStuff(PrintWriter pw, HTTPRequest httpRequest){
            
        // tulostetaan testausinformaatio jos asetuksissa on niin mritelty            
        if (printIt){
            pw.println("\n\n <!-- Testing stuff -->");
            pw.println("<div style='border: dotted 1px; margin: 2em; padding: 1em;'>");
            
            // tulostetaan kaikki saadut parametrit
    
            String[] parameterNames = httpRequest.getFormVariableNames();
    
            pw.print("<p>HTTP-pyynnn mukana tulleet parametrit: <br>\n");
            for (int i = 0; i < parameterNames.length; i++){
                pw.print(parameterNames[i] + " => " +
                     httpRequest.getFormVar(parameterNames[i])+"<br>");
            }
    
            // tulostetaan kaikki selaimelta saadut keksit
            // (itse asiassa tll hetkell vain session-id niminen keksi)
    
            String[] cookieNames = httpRequest.getCookieNames();
    
            pw.print("<p>Selaimen lhettmt keksit: <br>\n");
    //        for (int i = 0; i < cookieNames.length; i++){
    //            pw.print(cookieNames[i] + " => " +
    //                     httpRequest.getCookie(parameterNames[i])+"<br>");
    //        }
            pw.print("session-id: " + httpRequest.getCookie("session-id"));
            pw.print("</p>\n");
            pw.println("</div>\n\n");
        }
    }
    
    
  /**
  * Muuttaa &lt; ja &gt; -merkit &amp;lt; ja &amp;gt; -merkeiksi
  */
  public static String convertTag2Entity(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("<", "&lt;");
    in = in.replaceAll(">", "&gt;");
    return in;
  }

  /**
  *Poistetaan &lt; ja &gt; -merkit
  */
  public static String removeTag(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("<", "");
    in = in.replaceAll(">", "");
    return in;
  }

  /**
  * poistaa " ja ' -merkit.
  */

  public static String removeQuot(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("\"", "");
    in = in.replaceAll("'", "");
    return in;
  }


  /**
  *Kutsutaan removeTag,  removeQuot ja removeSpace -metodeja.
  *
  */
  public static String normalize(String in) {
    in = removeTag(in);
    in = removeQuot(in);
    in = removeSpace(in);
    return in;
  }

  /**
  * Poistaa vlilynnit ja rivinvaihdot yms.
  */

  public static String removeSpace(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("\\s", "");
    return in;
  }

  /**
  * Muuttaa vlilynnit ja rivinvaihdot yms.
  * alaviivoiksi
  */

  public static String convertSpace(String in) {
    if (in == null)
      return null;
    in = in.replaceAll("\\s", "_");
    return in;
  }

  /**
  Pohjelmametodi testausta varten.
  */
  public static void main(String[] args) {
    String html = "<tag>\"hello world!\" 'char'</tag>";
    System.out.println(convertTag2Entity(html));
    System.out.println(removeQuot(html));
    System.out.println(removeSpace(html));
    System.out.println(convertSpace(html));
  }

}
