package webcore.utilities;

/***
 * Simple Vector subclass which has convenience functions which treat Vector
 * elements as Strings.  Elements may still be Objects of any type.<p>
 * Vector elements are converted to String using the toString() method.  If the
 * element is null, the string "(null)" is used.
 */
public class StringVector extends java.util.Vector
{
	/****
     * Creates a new StringVector initialized with elements
     */
	public StringVector(String elements[])
 	{
  		for (int i=0;i<elements.length;i++)
    	{
     		add(elements[i]);
     	}
  	}

	/****
     * Creates a new StringVector initialized with elements
     */
	public StringVector(java.util.Collection collection)
 	{
      	super(collection);
  	}

   	/****
     * Creates a new, empty StringVector
     */
   	public StringVector()
    {

    }



   	/****
     * converts the Vector to an array of Strings
     */
	public synchronized String[] toStringArray()
 	{
        String result[] = new String[elementCount];
        for (int i=0;i<elementCount;i++)
        {
        	result[i] = getStringAt(i);
        }
   		return result;
    }

    /****
     * gets an element as a String
     */
    public String getStringAt(int index)
    {
    	return elementData[index]!=null?elementData[index].toString():"(null)";
    }

    /*****
     * returns a comma separated list of the Vector elements converted to
     * Strings.
     */
    public String toString()
    {
   		return toString("");
    }

    /*****
     * returns a comma separated list of the Vector elements converted to
     * Strings using the specified delimiter.
     */
    public String toString(String delimiter)
    {
        StringBuffer result = new StringBuffer();
        for (int i=0;i<elementCount;i++)
        {
        	if (result.length()> 0) result.append(",");
        	result.append(delimiter + getStringAt(i) + delimiter);
        }
   		return result.toString();
    }

}
