package webcore.webserver;

import java.net.*;
import java.lang.Runnable;
import java.io.*;


/****
 * WebRequestable to fulfill requests for static files.  The file is read
 * and returned to the client.  The mimetype of the file is determined by
 * extension using the MimeType class.
 * @see
 * MimeType
 */
public class HTTPFileRetriever implements WebRequestable
{

    /****
     * the default page displayed
     */
    public static String defaultDoc = "index.html";

    /****
     * the page to display when the requested page cannot be found
     */
    public static String notFoundDoc = "notfound.html";

    public void processRequest(HTTPRequest httpRequest) throws Exception
    {
        String fileStr = httpRequest.getRequestedPath();
        if (!webcore.utilities.FileUtils.doesFileExist(fileStr))
        {
            fileStr = httpRequest.getDocBase() + File.separator +
                notFoundDoc;
        }
        sendFile(httpRequest.getRequestSocket(), fileStr);
    }

    public void sendFile(Socket soc, String fileStr)
        throws java.io.IOException, java.io.FileNotFoundException
    {
        File requestedFile = new File(fileStr);
        String mimeCode;
        if (requestedFile.isDirectory())
        {
            requestedFile = new File(fileStr, defaultDoc);
            mimeCode = MimeType.getMimeType(defaultDoc);
        }
        else mimeCode = MimeType.getMimeType(fileStr);
        String strHeader = new String("HTTP/1.0 200 OK\r\nContent-type: " +
                         mimeCode);
        OutputStream outStream = null;
        try
        {
            outStream = soc.getOutputStream();
            byte buffer [] = new byte[1024 * 10];
            if (requestedFile.isFile() && fileStr.indexOf("..") < 0)
            {
                FileInputStream fileInput = new FileInputStream(requestedFile);
                int fileLen = (int)requestedFile.length();
                outStream.write((strHeader + "\r\nContent-length: " + fileLen +
                    "\r\n\r\n").getBytes());
                int amtread = 1;
                while (amtread > 0)
                {
                    amtread = fileInput.read(buffer);
                    if (amtread > 0)
                        outStream.write(buffer, 0, amtread);
                }
                fileInput.close();
            }
            else outStream.write((htmlHeader +
                "<BODY BGCOLOR=FFFFFF><H4>File not found: " +
                requestedFile.getName() + " </H4>").getBytes());
        }
        finally
        {
            outStream.flush();
            outStream.close();
        }
    }


}