package webcore.webserver;

/***
 * interface for all requestable objects via the webserver.
 * ServerSide java must implement this interface to process http requests.
 */
public interface WebRequestable
{

    /***
     * JavaScript entry to be added into FORM tags to make the forms execute
     * the sumittedform() JavaScript function
     */
    public final static String submitjavascripttag =
        "OnSubmit=\"return submittedform()\"";

    /***
     * JavaScript for the function: submittedform() which will be executed when
     * a form is submitted.  This text should be output before the BODY tag in
     * the HTML code.  The submittedform() JavaScript function makes sure a
     * FORM only gets submitted once, and displays an error dialog if an attempt
     * is made to submit a second time.
     */
    public final static String submitjavascript =
        "<SCRIPT>\r\n" +
        "<!--- Hide from old browsers\r\n" +
        "var wassubmitted=0\r\n" +
        "function submittedform() {\r\n" +
        "     if (wassubmitted==0)\r\n" +
        "     {\r\n" +
        "       wassubmitted=1;\r\n" +
        "       return true;\r\n" +
        "     }\r\n" +
        "       else\r\n" +
        "     {\r\n" +
        "       alert(\"You already submitted once.\");\r\n" +
        "       wassubmitted=1;\r\n" +
        "       return false;\r\n" +
        "     }\r\n" +
        "}\r\n" +
        "//end hiding from old browsers -->\r\n" +
        "</SCRIPT>\r\n";

    /**
     * The first line of an HTTP header response
     */
    public final static String httpBaseHeader =
        "HTTP/1.0 200 OK\r\n";

    /**
     * The header for an HTTP response with the Content-type of text/html.
     * It has one (CR-LF) after the Content-type tag.  Use this constant when
     * responding to clients if you want to add more HTTP information to the
     * header.  Otherwise, use htmlHeader.
     */
    public final static String htmlBaseHeader =
        httpBaseHeader + "Content-type: text/html\r\n";

    /**
     * The header for an HTTP response for HTML with a Content-type of
     * text/html.
     * It has two (CR-LF) after the Content-type tag.  After outputing this
     * constant to the client, the client is ready for the data part of the
     * HTTP response.  (That is, the header is complete.)
     */
    public final static String htmlHeader =
        htmlBaseHeader + "\r\n";

    /***
     * for backward compatability
     */
    public final static String httpHeader = htmlHeader;


    /**
     * The header for an HTTP response for plain text, with a Content-type of
     * text/plain.
     * It has one (CR-LF) after the Content-type tag.  Use this constant when
     * responding to clients if you want to add more HTTP information to the
     * header.  Otherwise, use textHeader.
     */
     public final static String textBaseHeader =
        httpBaseHeader + "Content-type: text/plain\r\n";

    /**
     * The header for an HTTP response with the Content-type of text/plain.
     * It has two (CR-LF) after the Content-type tag.  After outputing this
     * constant to the client, the client is ready for the data part of the
     * HTTP response.  (That is, the header is complete.)
     */
    public final static String textHeader = textBaseHeader + "\r\n";

    /***
     * This method is called by the Webserver when the WebRequestable is
     * requested by a client.
     */
    public void processRequest(HTTPRequest request) throws Exception;
}