
package webcore.webserver.application;


import webcore.webserver.*;
import converge.kernel.*;

/****
 * Application to start a webserver.
 
 Joni: listty mahdollisuus antaa porttinumerot parametrina main-metodille
 */

public class WWWServerConsole implements WebServerListener
{

    /**
     * Starts the web server.
     *
     * <pre>
     *     Usage: documentBase, REDIRECT | ssldocumentBase keystorePath keystorePass
     * 			| documentBase ssldocumentBase keystorePath keystorePass
     * </pre>
     *
     * documentBase is the path to the document base.<br>
     *
     * ssldocumentBase is the document base for ssl server<br>
     *
     * if 3 arguments are provided, the ssl server starts<br>
     *
     * if 4 arguments are provided, both the non-ssl and ssl servers are
     * started. (If documentBase is the text "REDIRECT" then the non-ssl
     * server will redirect browsers to the https server.)
     *
     * <p>
     * <b>Note:</b>java keystores allow separate keystore key passwords,
     * this application expects the keystore password and all key passwords
     * to be the same.
     */
    static public void main(String [] args)
    {
        if( args.length == 1 )
        {
            System.out.println("Starting with non-SSL");
            new WWWServerConsole( args[0] );
        }
        else if( args.length == 3 )
        {
            System.out.println("Starting with SSL");
            new WWWServerConsole( args[0], args[1], args[2] );
        }
        else if( args.length == 4 )
        {
            System.out.println("Starting with SSL and non-SSL");
            new WWWServerConsole( args[0], args[1], args[2], args[3]);
        }
		
		// joni: 
		// 5. ja 6. parametri mrittvt nyt kytettvt porttinumerot
		
		else if ( args.length == 6 ){
			System.out.println("Starting with SSL and non-SSL");
			int port, sslPort;
			try{
				port = Integer.parseInt(args[4]);
				sslPort = Integer.parseInt(args[5]);
			}
			catch(Exception e){
				System.out.println("WWWSERVERCONSOLE: " + e);
				port = 0;		// kytetn oletuksia, ks. tarkistus alempana
				sslPort = 0;
			}
			
			// kynnistetn asetustiedostossa mriteltyihin portteihin
			// tai jos ei onnistu, niin 8080 ja 8081
			
			if ( port >= 1024 && sslPort >= 1024 && port != sslPort && 
				port <= 65535 && sslPort <= 65535)
				new WWWServerConsole( args[0], args[1], args[2], args[3], port, sslPort);
			else
				new WWWServerConsole( args[0], args[1], args[2], args[3]);					
		}
        else
        {
            System.out.println("Usage: documentBase,REDIRECT | ssldocumentBase " +
                "keystorePath keystorePass | documentBase ssldocumentBase " +
                "keystorePath keystorePass");
            System.exit(1);
        }
    }

	/**
	Apumetodi joka kutsuu main-metodia
		
	*/
	public static void start(String[] args, int port, int SSLport){
	
		// ...
		
	}


    public WWWServerConsole(String documentBase,
        String sslDocumentBase, String ksPath, String pass)
    {
        try
        {
            if(ksPath != null && pass != null &&
                sslDocumentBase != null)
            {
                WebServer sslws = new WebServer();
                sslws.setPort(8081);
                WebServerSSLSocketFactory sslSocketFactory =
                    new WebServerSSLSocketFactory(ksPath, pass);
                sslws.setSocketFactory(sslSocketFactory);
                sslws.setDocumentBase(sslDocumentBase);
                sslws.addWebServerListener(this);
                sslws.start();
            }

            if (documentBase != null)
            {
                WebServer ws = new WebServer();
                ws.setPort(8080);
                if (documentBase.equals("REDIRECT"))
                    ws.setWebRequestableFactory(
                        new SSLRedirectorWebRequestableFactory());
                else ws.setDocumentBase(documentBase);
                ws.addWebServerListener(this);
                ws.start();
            }
        }
        catch (Exception e)
        {
            System.err.println(e);
            System.err.println("Shutting application");
            System.exit(1);
        }
    }

	
		/**
		Toinen konstruktori jolle voi antaa porttinumerot parametrina
		*/
	    public WWWServerConsole(String documentBase,
        String sslDocumentBase, String ksPath, String pass, int port, int sslPort)
    {
        try
        {
            if(ksPath != null && pass != null &&
                sslDocumentBase != null)
            {
                WebServer sslws = new WebServer();
                sslws.setPort(sslPort);
                WebServerSSLSocketFactory sslSocketFactory =
                    new WebServerSSLSocketFactory(ksPath, pass);
                sslws.setSocketFactory(sslSocketFactory);
                sslws.setDocumentBase(sslDocumentBase);
                sslws.addWebServerListener(this);
                sslws.start();
            }

            if (documentBase != null)
            {
                WebServer ws = new WebServer();
                ws.setPort(port);
                if (documentBase.equals("REDIRECT"))
                    ws.setWebRequestableFactory(
                        new SSLRedirectorWebRequestableFactory());
                else ws.setDocumentBase(documentBase);
                ws.addWebServerListener(this);
                ws.start();
            }
        }
        catch (Exception e)
        {
            System.err.println(e);
            System.err.println("Shutting application");
            System.exit(1);
        }
    }
	
	
	
    public WWWServerConsole(String documentBase)
    {
        this(documentBase,null,null,null);
    }

    public WWWServerConsole(String sslDocumentBase, String ksPath, String pass)
    {
        this(null,sslDocumentBase,ksPath,pass);
    }

    public void startedListening(WebServerEvent e)
    {
        System.out.println(e.getArg());
    }

    public void stoppedListening(WebServerEvent e)
    {
        System.out.println(e.getArg());
    }

    public void minorErrorOccured(WebServerEvent e)
    {
        //Ignore minor errors

    }

    private String extractName(String classname)
    {
        int i = classname.lastIndexOf('.');
        if (i > 0) return classname.substring(i+1);
        else return classname;
    }

    public void requested(WebServerEvent e)
    {
        HTTPRequest req = (HTTPRequest)e.getArg();

        Log.output("webcore").println(
            req.getRequestSocket().getInetAddress().getHostAddress() +
            " [" + new java.util.Date().toString() + "]" +
            "[" + req.getRequestedFile() + "][" + extractName(
            req.getRequestSocket().getClass().getName()) + "]" );

    }

}





